package roneditor;

import java.util.ArrayList;
import java.util.EventObject;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.Plugin;
import org.eclipse.core.runtime.Preferences;
import org.eclipse.draw2d.ConnectionLayer;
import org.eclipse.draw2d.FigureCanvas;
import org.eclipse.draw2d.MarginBorder;
import org.eclipse.draw2d.Viewport;
import org.eclipse.draw2d.parts.ScrollableThumbnail;
import org.eclipse.draw2d.parts.Thumbnail;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.EditPartFactory;
import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.KeyHandler;
import org.eclipse.gef.KeyStroke;
import org.eclipse.gef.LayerConstants;
import org.eclipse.gef.Tool;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.gef.commands.CommandStackListener;
import org.eclipse.gef.editparts.ScalableFreeformRootEditPart;
import org.eclipse.gef.editparts.ZoomManager;
import org.eclipse.gef.palette.PaletteRoot;
import org.eclipse.gef.tools.MarqueeSelectionTool;
import org.eclipse.gef.ui.actions.ActionBarContributor;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.gef.ui.actions.GEFActionConstants;
import org.eclipse.gef.ui.actions.RedoAction;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.gef.ui.actions.UndoAction;
import org.eclipse.gef.ui.actions.UpdateAction;
import org.eclipse.gef.ui.actions.WorkbenchPartAction;
import org.eclipse.gef.ui.actions.ZoomComboContributionItem;
import org.eclipse.gef.ui.actions.ZoomInAction;
import org.eclipse.gef.ui.actions.ZoomOutAction;
import org.eclipse.gef.ui.palette.FlyoutPaletteComposite;
import org.eclipse.gef.ui.palette.PaletteViewer;
import org.eclipse.gef.ui.palette.PaletteViewerProvider;
import org.eclipse.gef.ui.parts.GraphicalEditorWithFlyoutPalette;
import org.eclipse.gef.ui.parts.ScrollingGraphicalViewer;
import org.eclipse.gef.ui.parts.SelectionSynchronizer;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IStatusLineManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.SubActionBars;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.actions.RetargetAction;
import org.eclipse.ui.part.IPage;
import org.eclipse.ui.part.IPageSite;
import org.eclipse.ui.part.Page;
import org.eclipse.ui.part.PageBookView;
import org.eclipse.ui.views.properties.IPropertySheetPage;

import roneditor.editor.ContextMenuProviderWithActionRegistry;

/**
 * <p>
 * This class is based roughly on {@link GraphicalEditorWithFlyoutPalette}.
 * </p>
 * 
 * <p>
 * It is an {@link IPage} that contains some {@link GraphicalViewer}s and a
 * {@link FlyoutPaletteComposite} and is able to change the current viewer. It
 * is meant to be used in a {@link PageBookView} (especially
 * {@link AbstractPageBookView}) like an editor. Like an editor this page has
 * its own {@link EditDomain} and {@link ActionRegistry}. A top model element
 * and a parent editor (from which this page is being called to show up) are set
 * in the constructor. A {@link ZoomManagerDelegate} for a
 * {@link ZoomComboContributionItem} is provided via {@link #getAdapter(Class)}
 * to the workbench page's part service.
 * </p>
 * 
 * <p>
 * <b>Prerequisites</b>: The parent editor must provide non-<code>null</code>
 * instances of the following classes via getAdapter():
 * <ul>
 * <li> a {@link CommandStack} which will be used for this page's
 * {@link EditDomain} and to update actions.
 * <li> a {@link Plugin} whose preferences will be used by the
 * {@link FlyoutPaletteComposite} to store states of the palette.
 * <li> a {@link SelectionSynchronizer} to synchronize this page's
 * {@link GraphicalViewer} with other {@link EditPartViewer}s that show parts
 * of the parent editor's model.
 * <li> a {@link KeyHandler} (optional, may be null) which will be set as parent
 * for the Keyhandler of this page's {@link GraphicalViewer}.
 * </ul>
 * </p>
 * 
 * <p>
 * When initialized and in {@link #createControl(Composite)} this class does the
 * following:
 * <ol>
 * <li> The page registers itself as an eAdapter of the model {@link EObject}
 * that can react on notifications of the EMF model. For this, subclasses may
 * implement
 * <li> The page registers itself as an {@link CommandStackListener} on the
 * editor's {@link CommandStack} to update actions on command stack changes.
 * <li> A {@link FlyoutPaletteComposite} is being created which uses the
 * {@link Preferences} of the {@link Plugin} delivered by the parent editor.
 * <li> Some {@link MultiViewerPageViewer}s (depending on
 * {@link #getNumberOfViewers()}) are being created and added to the
 * {@link SelectionSynchronizer} delivered by the parent editor. The objects
 * returned by {@link #getViewerContents()} are set as contents of these
 * viewers. <br>
 * <li> A {@link Thumbnail} showing a miniature overview of the current viewer
 * is being installed below the palette.
 * <li> The first viewer is set as the current viewer.
 * <li> Actions are being created: {@link ZoomInAction}, {@link ZoomOutAction}
 * and custom Actions in {@link #createActions()}. Some standard handles for
 * {@link RetargetAction}s from the {@link ActionFactory} besides zooming and
 * undo/redo are being registered if they have been created in
 * {@link #createActions()}: COPY, CUT, PASTE, DELETE. <br>
 * </ol>
 * </p>
 * 
 * <p>
 * A {@link MultiViewerPageViewer} notices if a mouse click occurs on it and
 * sets itself as the current viewer which performs the following updates:
 * <ol>
 * <li> The clicked viewer is set as {@link ISelectionProvider} of the
 * {@link IPageSite}.
 * <li> The page registers itself as an {@link ISelectionChangedListener} on the
 * viewer to be able to update the {@link UpdateAction}s in its
 * {@link ActionRegistry} on selection changes.
 * <li> The viewer is set as selection provider for all {@link SelectionAction}s
 * in the {@link ActionRegistry}.
 * <li> The {@link Thumbnail} is being configured to show the new current
 * viewer.
 * <li> The {@link ZoomManager} of the clicked viewer is set as delegating
 * target in the {@link ZoomManagerDelegate}.
 * </ol>
 * Ths contents of an existing viewer may be replaced with another object via
 * {@link #setViewersContents(int, EObject)}.
 * </p>
 * 
 * <p>
 * These methods have to be implemented by subclasses:
 * <ul>
 * <li> {@link #createActions()}: Actions may be registered with
 * {@link #getActionRegistry()}.<br>
 * Notice that the current viewer will be set automatically as selection
 * provider for all {@link SelectionAction}s if it changes!
 * <li> {@link #createContextMenuProvider()}: A
 * {@link ContextMenuProviderWithActionRegistry} for the context menu of each
 * viewer.
 * <li> {@link #createEditPartFactory()}: A suitable {@link EditPartFactory}
 * for the model elements.
 * <li> {@link #createPaletteRoot()}: A {@link PaletteRoot} defining what the
 * palette should contain.
 * <li> {@link #getNumberOfViewers()} specifies how many viewers will be created
 * on the page.
 * <li> {@link #getThumbSashWeights()}: Must return an array of two integers
 * describing the ratio of how the flyout composite's space is divided between
 * the palette and the thumbnail.
 * <li> {@link #getViewerContents()} designates the objects that will be set as
 * contents for the viewers. This has to conform to
 * {@link #getNumberOfViewers()}.
 * <li> {@link #setupKeyHandler(KeyHandler)}: In this method {@link KeyStroke}s
 * may be associated with previously defined Actions in the passed
 * {@link KeyHandler} which is the {@link GraphicalViewer}'s one.
 * <li>
 * {@link Adapter#notifyChanged(org.eclipse.emf.common.notify.Notification)}:
 * This is optional, for reacting on model change notifications. May be just an
 * empty method.
 * </ul>
 * </p>
 * 
 * @author Tony Modica
 */
public abstract class AbstractMultiViewerPage extends Page implements
		ISelectionChangedListener, IAdaptable, Adapter, CommandStackListener {

	/**
	 * The action registry that hold the actions for this page.
	 */
	private final ActionRegistry actionRegistry = new ActionRegistry();

	/**
	 * The main {@link Control} of this page.
	 */
	private FlyoutPaletteComposite flyoutPaletteComposite;

	/**
	 * The editor that requested to display the model.
	 */
	private final IEditorPart editor;

	/**
	 * An array of the viewers hosted by this page. Its length is specified by
	 * {@link #getNumberOfViewers()}.
	 * 
	 */
	private final MultiViewerPageViewer[] viewers = new MultiViewerPageViewer[getNumberOfViewers()];

	/**
	 * The viewer that is
	 * <ul>
	 * <li> showed in the {@link Thumbnail},
	 * <li> the current {@link ISelectionProvider} for this page's
	 * {@link IPageSite} and
	 * <li> providing a {@link ZoomManager} for the zoom actions via a
	 * {@link ZoomManagerDelegate}.
	 * </ul>
	 * 
	 * @see #setCurrentViewer(roneditor.AbstractMultiViewerPage.MultiViewerPageViewer)
	 */
	private MultiViewerPageViewer currentViewer;

	/**
	 * A thumbnail showing a miniature of what the viewer contains.
	 */
	private ScrollableThumbnail thumbnail;

	/**
	 * The {@link EditDomain} managing this page like an encapsulated editor.
	 */
	private EditDomain editDomain;

	/**
	 * The model which is shown in the viewer.
	 */
	private final EObject model;

	/**
	 * Target field for {@link Adapter} implementation.
	 */
	private Notifier target;

	/**
	 * This subclassed {@link ZoomManager} is set for the zoom actions and the
	 * returned by {@link #getAdapter(Class)} for the
	 * {@link ZoomComboContributionItem} created in the
	 * {@link ActionBarContributor}. It delegates to the {@link ZoomManager} of
	 * {@link #currentViewer}.
	 */
	private final ZoomManagerDelegate zoomManagerDelegate = new ZoomManagerDelegate();

	/**
	 * This enables this page to switch the actual selection-providing viewer
	 */
	private final SelectionProviderIntermediate selectionProvider = new SelectionProviderIntermediate();

	/**
	 * A shared {@link KeyHandler} for the viewers on this page.
	 */
	private KeyHandler keyHandler;

	/**
	 * @param model
	 *            The model element to be displayed in this page's
	 *            {@link GraphicalViewer}.
	 * @param editor
	 *            The editor that requested the model to be displayed.
	 */
	protected AbstractMultiViewerPage(final EObject model,
			final IEditorPart editor) {
		this.editor = editor;
		this.model = model;
	}

	/**
	 * @return The model passed in the constructor.
	 */
	public EObject getModel() {
		return model;
	}

	/**
	 * Hooks into the model.
	 */
	@Override
	public void init(final IPageSite pageSite) {
		super.init(pageSite);
		getModel().eAdapters().add(this);
		((CommandStack) getEditor().getAdapter(CommandStack.class))
				.addCommandStackListener(this);
	}

	/**
	 * Creates and registers actions. Handle actions for some standard
	 * {@link RetargetAction}s are set:
	 * <ul>
	 * <li> {@link ZoomInAction} and {@link ZoomOutAction} (with
	 * {@link #zoomManagerDelegate}),
	 * <li> copy, cut, paste, delete, undo, redo from the GEF
	 * {@link ActionFactory},
	 * </ul>
	 * 
	 * <p>
	 * Except for zoom, undo, and redo, these handles have to be created by
	 * subclasses via {@link #createActions()} and with appropriate IDs.
	 * </p>
	 */
	private void installActions() {
		createActions();

		// create standard zoom actions
		IAction action;
		action = new ZoomInAction(zoomManagerDelegate);
		getActionRegistry().registerAction(action);
		// GEFActionConstants.ZOOM_IN is id of ZoomInRetargetAction

		action = new ZoomOutAction(zoomManagerDelegate);
		getActionRegistry().registerAction(action);
		// GEFActionConstants.ZOOM_OUT is id of ZoomOutRetargetAction

		// create undo and redo actions
		action = new UndoAction(getEditor());
		getActionRegistry().registerAction(action);
		action = new RedoAction(getEditor());
		getActionRegistry().registerAction(action);

		// Register handles for global RetargetActions
		registerHandleForEditorAction(ActionFactory.COPY.getId());
		registerHandleForEditorAction(ActionFactory.CUT.getId());
		registerHandleForEditorAction(ActionFactory.PASTE.getId());
		registerHandleForEditorAction(ActionFactory.DELETE.getId());
		registerHandleForEditorAction(ActionFactory.UNDO.getId());
		registerHandleForEditorAction(ActionFactory.REDO.getId());
		registerHandleForEditorAction(GEFActionConstants.ZOOM_IN);
		registerHandleForEditorAction(GEFActionConstants.ZOOM_OUT);
	}

	/**
	 * To be implemented by subclasses. Actions have to be registered via
	 * {@link #getActionRegistry()}. <br>
	 * {@link WorkbenchPartAction}s should get the editor ({@link #getEditor()})
	 * as workbench part in their constructor. <br>
	 * To register actions as handles for {@link RetargetAction}s you may use
	 * {@link #registerHandleForEditorAction(String)} with a string that matches
	 * both the {@link RetargetAction}'s and the handle's ID.
	 * 
	 * <p>
	 * In
	 * {@link #setCurrentViewer(roneditor.AbstractMultiViewerPage.MultiViewerPageViewer)}
	 * the {@link #currentViewer} will be set with
	 * {@link SelectionAction#setSelectionProvider(ISelectionProvider)} for
	 * {@link SelectionAction}s.
	 * </p>
	 * 
	 * @see #installActions() to find out which actions will be created by
	 *      default.
	 */
	abstract protected void createActions();

	/**
	 * Registers an Action as a handler for a RetargetAction. This has to be
	 * done only once since
	 * {@link PageBookView#partActivated(org.eclipse.ui.IWorkbenchPart)} takes
	 * care about refreshing the global action handlers for the ViewSite with
	 * the global action handlers defined for the {@link SubActionBars} of this
	 * {@link IPageSite}.
	 */
	protected void registerHandleForEditorAction(final String id) {
		getSite().getActionBars().setGlobalActionHandler(id,
				getActionRegistry().getAction(id));
	}

	/**
	 * This method creates all the visual parts and configures them:
	 * 
	 * <ol>
	 * <li> A {@link FlyoutPaletteComposite} is created and associated with the
	 * {@link Preferences} of the {@link #editor}'s {@link Plugin}.
	 * <li> With {@link #createGraphicalViewerComposite()} a composite hosting
	 * the {@link MultiViewerPageViewer}s is being created and set as graphical
	 * control for the {@link #flyoutPaletteComposite}.
	 * <li> The {@link Thumbnail} is being created with
	 * {@link #installThumbnailInPalette()}.
	 * <li> The first viewer in {@link #viewers} is set with
	 * {@link #setCurrentViewer(roneditor.AbstractMultiViewerPage.MultiViewerPageViewer)}.
	 * <li> Actions are being created via {@link #installActions()}.
	 * </ol>
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.Page#createControl(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	public void createControl(final Composite parent) {
		final Plugin plugin = (Plugin) getEditor().getAdapter(Plugin.class);
		Assert.isNotNull(plugin,
				"The editor did not deliver a plugin instance for the page!");
		flyoutPaletteComposite = new FlyoutPaletteComposite(parent, SWT.NONE,
				getSite().getPage(),
				new PaletteViewerProvider(getEditDomain()),
				FlyoutPaletteComposite.createFlyoutPreferences(plugin
						.getPluginPreferences()));

		// This sets the state of the flyout palette to "pinned open"
		plugin.getPluginPreferences().setValue("org.eclipse.gef.pstate", 4);

		// create control that hosts graphical viewer
		final Composite viewerControl = createGraphicalViewerComposite();
		flyoutPaletteComposite.setGraphicalControl(viewerControl);

		// install thumbnail
		installThumbnailInPalette();

		// set first viewer as default current viewer
		setCurrentViewer(getViewers()[0]);

		// set selection provider intermediate for the page's site
		getSite().setSelectionProvider(selectionProvider);

		// create actions, this is done here because zoom actions need a
		// viewer's zoom manager
		installActions();
	}

	/**
	 * @return A lazily created {@link EditDomain} with slightly modified
	 *         selection behaviour.
	 */
	private EditDomain getEditDomain() {
		if (null == editDomain) {
			editDomain = new EditDomain() {
				/**
				 * Overridden to set different behaviour for the
				 * {@link MarqueeSelectionTool}.
				 * 
				 * @see org.eclipse.gef.EditDomain#setActiveTool(org.eclipse.gef.Tool)
				 */
				@Override
				public void setActiveTool(final Tool tool) {
					if (tool instanceof MarqueeSelectionTool) {
						((MarqueeSelectionTool) tool)
								.setMarqueeBehavior(MarqueeSelectionTool.BEHAVIOR_NODES_AND_CONNECTIONS);
					}
					super.setActiveTool(tool);
				}
			};
			editDomain.setPaletteRoot(createPaletteRoot());
			final CommandStack commandStack = (CommandStack) getEditor()
					.getAdapter(CommandStack.class);
			Assert
					.isNotNull(commandStack,
							"The editor did not deliver a command stack instance for the page!");
			editDomain.setCommandStack(commandStack);
		}
		return editDomain;
	}

	/**
	 * Subclasses must implement to specify the {@link PaletteRoot} that should
	 * be used by this page's {@link EditDomain} and which defines the contents
	 * of the palette.
	 * 
	 * @return The {@link PaletteRoot} for the {@link EditDomain} of this page.
	 */
	abstract protected PaletteRoot createPaletteRoot();

	/**
	 * Creates a {@link Thumbnail} on the {@link Control} of this page's
	 * {@link PaletteViewer}.
	 * 
	 * @see #getThumbSashWeights()
	 */
	private void installThumbnailInPalette() {
		// get the FigureCanvas of the PaletteViewer
		final FigureCanvas figureCanvas = (FigureCanvas) getEditDomain()
				.getPaletteViewer().getControl();

		// install a SashForm on the parent of the FigureCanvas
		final SashForm sashForm = new SashForm(figureCanvas.getParent(),
				SWT.VERTICAL);

		// move the original figureCanvas onto the SashForm
		figureCanvas.setParent(sashForm);

		// create a new FigureCanvas and put a new Thumbnail on it
		final FigureCanvas thumbCanvas = new FigureCanvas(sashForm);
		thumbnail = new ScrollableThumbnail();
		thumbnail.setBorder(new MarginBorder(3));
		thumbCanvas.setContents(thumbnail);

		// adjust the position of the thumbnail on the canvas
		sashForm.setWeights(getThumbSashWeights());
	}

	/**
	 * The two values in the returned array specify how the flyout composite's
	 * space is being divided between the palette entries and the thumbnail in
	 * the palette. When implementing this template should be used: <br>
	 * <code> return new int[] { Palette_Weight, Thumb_Weight }; </code>
	 * 
	 * @return An array of 2 integers describing the ratio of the palette to the
	 *         thumbnail.
	 */
	abstract protected int[] getThumbSashWeights();

	/**
	 * This method creates an configures some {@link MultiViewerPageViewer}
	 * according to {@link #getNumberOfViewers()} and adds them to a
	 * {@link SashForm}:
	 * 
	 * <p>
	 * For each newly created graphical viewer
	 * 
	 * <ol>
	 * <li> the shared {@link #editDomain} is set,
	 * <li> a shared {@link KeyHandler} (configured via
	 * {@link #setupKeyHandler(KeyHandler)}) is set,
	 * <li> it is registered with the {@link #editor}'s
	 * {@link SelectionSynchronizer},
	 * <li> a shared {@link EditPartFactory} returned by
	 * {@link #createEditPartFactory()} is set,
	 * <li> a unique {@link ContextMenuProviderWithActionRegistry} returned by
	 * {@link #createContextMenuProvider(EditPartViewer)},
	 * <li> an object of the array {@link #getViewerContents()} returns is set
	 * as the contents of the viewers. This depends on the viewers' position.
	 * <li> The {@link ZoomManager} of the viewer is configured with some zoom
	 * levels.
	 * </ol>
	 * </p>
	 * 
	 * @return The {@link SashForm} composite holding all the viewers.
	 */
	private Composite createGraphicalViewerComposite() {
		// create composite that hold the graphical viewers
		final Composite viewerComposite = new SashForm(flyoutPaletteComposite,
				SWT.BORDER);
		viewerComposite.setLayout(new FillLayout());
		viewerComposite.setBackground(flyoutPaletteComposite.getDisplay()
				.getSystemColor(SWT.COLOR_LIST_BACKGROUND));
		viewerComposite.setForeground(flyoutPaletteComposite.getDisplay()
				.getSystemColor(SWT.COLOR_LIST_FOREGROUND));
		viewerComposite.setLayoutData(new GridData(GridData.FILL_BOTH));

		// prepare shared edit part factory
		final EditPartFactory editPartFactory = createEditPartFactory();

		// prepare contents to set
		final EObject[] contents = getViewerContents();
		Assert
				.isTrue(
						0 < getNumberOfViewers(),
						"AbstractMultiViewerPages must demand at least 1 viewer in getNumberOfViewers()!");
		Assert.isTrue(contents.length == getNumberOfViewers(), contents.length
				+ " contents objects specified, but " + getNumberOfViewers()
				+ " viewers are demanded by AbstractMultiViewerPage!");

		// create viewers
		for (int i = 0; i < getNumberOfViewers(); i++) {
			final MultiViewerPageViewer viewer = new MultiViewerPageViewer();

			// set the new viewer on the viewer composite
			viewer.createControl(viewerComposite);

			getViewers()[i] = viewer;
			viewer.setEditDomain(getEditDomain());
			getEditDomain().addViewer(viewer);
			viewer.setKeyHandler(getKeyHandler());
			getSelectionSynchronizer().addViewer(viewer);
			final ScalableFreeformRootEditPart rootEditPart = new ScalableFreeformRootEditPart();
			viewer.setRootEditPart(rootEditPart);
			viewer.setEditPartFactory(editPartFactory);
			viewer.setContents(getViewerContents()[i]);
			viewer.setContextMenu(createContextMenuProvider(viewer));

			// set antialias on connection layer, this may be turned off if
			// causing trouble
			((ConnectionLayer) rootEditPart
					.getLayer(LayerConstants.CONNECTION_LAYER))
					.setAntialias(SWT.ON);

			// initialize viewer's ZoomManager
			final ZoomManager zoomManager = (ZoomManager) viewer
					.getProperty(ZoomManager.class.toString());
			final List<String> zoomLevels = new ArrayList<String>(3);
			zoomLevels.add(ZoomManager.FIT_ALL);
			zoomLevels.add(ZoomManager.FIT_WIDTH);
			zoomLevels.add(ZoomManager.FIT_HEIGHT);
			zoomManager.setZoomLevelContributions(zoomLevels);
			zoomManager.setZoomAnimationStyle(ZoomManager.ANIMATE_ZOOM_IN_OUT);

			// some optional coloring
			viewer.getControl().setBackground(
					PlatformUI.getWorkbench().getDisplay().getSystemColor(
							SWT.COLOR_LIST_BACKGROUND));
			viewer.getControl().setForeground(
					PlatformUI.getWorkbench().getDisplay().getSystemColor(
							SWT.COLOR_LIST_FOREGROUND));
		}
		return viewerComposite;
	}

	/**
	 * @return A lazily created shared {@link KeyHandler} for the viewers on
	 *         this page.
	 */
	protected KeyHandler getKeyHandler() {
		if (keyHandler == null) {
			keyHandler = new KeyHandler();
			// this may be null
			keyHandler.setParent((KeyHandler) getEditor().getAdapter(
					KeyHandler.class));
			keyHandler
					.put(KeyStroke.getPressed(SWT.DEL, 127, 0),
							getActionRegistry().getAction(
									ActionFactory.DELETE.getId()));
			keyHandler.put(KeyStroke.getPressed(SWT.F2, 0), getActionRegistry()
					.getAction(GEFActionConstants.DIRECT_EDIT));
			// add key strokes defined by concrete subclasses
			setupKeyHandler(keyHandler);
		}
		return keyHandler;
	}

	/**
	 * @return The number of viewers that shall be created on this page.
	 * 
	 * @see #getViewerContents()
	 * @see #createGraphicalViewerComposite()
	 */
	abstract public int getNumberOfViewers();

	/**
	 * The viewer's are arranged horizontal and will be showing these objects in
	 * the order of the returned array from left to right. The length of the
	 * array has to be exactly the value of {@link #getNumberOfViewers()}!
	 * 
	 * @return The objects that will be set as contents in each of
	 *         {@link #viewers}.
	 * 
	 * @see #setViewersContents(int, EObject)
	 * @see #createGraphicalViewerComposite()
	 */
	abstract public EObject[] getViewerContents();

	/**
	 * A method to access the object which is being displayed in a specific
	 * viewer.
	 * 
	 * @param viewerPosition
	 *            the position (starting with 0) of the viewer
	 * @return the model of the top edit part of viewers[viewerPosition]
	 * @see #setViewersContents(int, EObject)
	 */
	public EObject getViewersContents(final int viewerPosition) {
		Assert.isTrue(viewerPosition < getNumberOfViewers(),
				"AbstractMultiViewerPage tried to retrieve contents for viewer on position"
						+ viewerPosition + " but has only "
						+ getNumberOfViewers() + "viewers!");
		return (EObject) getViewers()[viewerPosition].getContents().getModel();
	}

	/**
	 * A method to set a new object to be displayed in a specific viewer.
	 * 
	 * @param viewerPosition
	 *            the position (starting with 0) of the viewer
	 * @see #getViewerContents()
	 */
	public void setViewersContents(final int viewerPosition, final EObject model) {
		Assert.isTrue(viewerPosition < getNumberOfViewers(),
				"AbstractMultiViewerPage tried to set contents for viewer on position"
						+ viewerPosition + " but has only "
						+ getNumberOfViewers() + "viewers!");
		getViewers()[viewerPosition].setContents(model);
	}

	/**
	 * This method inverts the visibility of a viewer's control and relayouts
	 * the SashForm that the visible viewers on it share the remaining space.
	 * 
	 * @param viewerPosition
	 *            the position (starting with 0) of the viewer
	 */
	public void switchViewerVisibility(final int viewerPosition) {
		Assert.isTrue(viewerPosition < getNumberOfViewers(),
				"AbstractMultiViewerPage tried to switch visibility of viewer on position"
						+ viewerPosition + " but has only "
						+ getNumberOfViewers() + "viewers!");
		final Control control = getViewers()[viewerPosition].getControl();
		control.setVisible(!control.isVisible());
		final SashForm sashForm = (SashForm) control.getParent();
		sashForm.layout(false);
	}

	/**
	 * Subclasses must implement to associate {@link KeyStroke}s with Actions
	 * in the passed {@link KeyHandler}} which is the one used for the
	 * {@link GraphicalViewers}.
	 * 
	 * @see #createGraphicalViewerComposite()
	 */
	abstract protected void setupKeyHandler(KeyHandler kh);

	/**
	 * Subclasses must implement to specify the
	 * {@link ContextMenuProviderWithActionRegistry} that should be used by this
	 * page's {@link GraphicalViewer}s and is responsible to show the created
	 * actions in the context menu.
	 * 
	 * @param viewer
	 *            The graphical viewer for context menu provider
	 * @return The {@link ContextMenuProviderWithActionRegistry} for the
	 *         specified {@link GraphicalViewer}.
	 * 
	 * @see #createGraphicalViewerComposite()
	 */
	abstract protected ContextMenuProviderWithActionRegistry createContextMenuProvider(
			EditPartViewer viewer);

	/**
	 * Subclasses must implement to specify the {@link EditPartFactory} that
	 * should be used by this page's {@link GraphicalViewer}s.
	 * 
	 * @return The {@link EditPartFactory} for the {@link GraphicalViewer}s of
	 *         this page.
	 * 
	 * @see #createGraphicalViewerComposite()
	 */
	abstract protected EditPartFactory createEditPartFactory();

	/**
	 * @return The {@link #editor}'s {@link SelectionSynchronizer}
	 */
	private SelectionSynchronizer getSelectionSynchronizer() {
		final SelectionSynchronizer synchronizer = (SelectionSynchronizer) getEditor()
				.getAdapter(SelectionSynchronizer.class);
		Assert
				.isNotNull(synchronizer,
						"The editor did not deliver a selection synchronizer instance for the page!");
		return synchronizer;
	}

	/**
	 * Unhooks the model, removes viewers from the {@link #editor}'s
	 * {@link SelectionSynchronizer} and deregisters this page as
	 * {@link #currentViewer}'s {@link ISelectionChangedListener} and
	 * deactivates the {@link #thumbnail}.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.Page#dispose()
	 */
	@Override
	public void dispose() {
		getModel().eAdapters().remove(this);

		for (final GraphicalViewer viewer : getViewers()) {
			getSelectionSynchronizer().removeViewer(viewer);
		}

		currentViewer.removeSelectionChangedListener(this);

		if (thumbnail != null) {
			thumbnail.deactivate();
			thumbnail = null;
		}
		getActionRegistry().dispose();
		((CommandStack) getEditor().getAdapter(CommandStack.class))
				.removeCommandStackListener(this);
		super.dispose();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.Page#getControl()
	 */
	@Override
	public Control getControl() {
		return flyoutPaletteComposite;
	}

	/**
	 * Sets focus to a part in the page.
	 */
	@Override
	public void setFocus() {
		if (getControl() != null) {
			getControl().setFocus();
		}
	}

	/**
	 * If a selection change occurs in the current viewer the
	 * {@link UpdateAction}s in the {@link ActionRegistry} are being updated.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.ISelectionChangedListener#selectionChanged(org.eclipse.jface.viewers.SelectionChangedEvent)
	 */
	public void selectionChanged(final SelectionChangedEvent event) {
		updateActions();
	}

	/**
	 * Updates all {@link UpdateAction}s registered in the
	 * {@link #actionRegistry}.
	 */
	protected void updateActions() {
		for (final Iterator<IAction> iter = getActionRegistry().getActions(); iter
				.hasNext();) {
			final IAction action = iter.next();
			if (action instanceof UpdateAction) {
				((UpdateAction) action).update();
			}
		}
	}

	/**
	 * Extended to provide {@link #zoomManagerDelegate} as {@link ZoomManager}.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	 */
	public Object getAdapter(final Class adapter) {
		if (adapter == ZoomManager.class) {
			return zoomManagerDelegate;
		} else if (adapter == IPropertySheetPage.class) {
			return editor.getAdapter(adapter);
		}
		return null;
	}

	/**
	 * @return the actionRegistry
	 */
	protected ActionRegistry getActionRegistry() {
		return actionRegistry;
	}

	/**
	 * @return the editor
	 */
	public IEditorPart getEditor() {
		return editor;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.emf.common.notify.Adapter#isAdapterForType(java.lang.Object)
	 */
	public boolean isAdapterForType(final Object type) {
		return type.equals(getModel().getClass());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.emf.common.notify.Adapter#setTarget(org.eclipse.emf.common.notify.Notifier)
	 */
	public void setTarget(final Notifier newTarget) {
		target = newTarget;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.emf.common.notify.Adapter#getTarget()
	 */
	public Notifier getTarget() {
		return target;
	}

	/**
	 * This method sets the current graphical viewer of this page which implies
	 * that the passed viewer
	 * 
	 * <ul>
	 * <li> will be shown in the {@link Thumbnail},
	 * <li> becomes the current {@link ISelectionProvider} for this page's
	 * {@link IPageSite} and
	 * <li> provides the {@link ZoomManager} for the zoom actions the
	 * {@link ZoomManagerDelegate} delegates to.
	 * </ul>
	 * 
	 * Additionally the page sets the current viewer as selection provider for
	 * its {@link SelectionAction}s by calling
	 * {@link #updateSelectionActionsProvider()} and registers itself as an
	 * {@link ISelectionChangedListener} on the new current viewer to be able to
	 * update its {@link UpdateAction}s
	 * 
	 * @param viewer
	 */
	protected void setCurrentViewer(final MultiViewerPageViewer viewer) {
		Assert.isNotNull(viewer,
				"AbstractMultiViewerPage can not be set to viewer 'null'!");

		// update current viewer only if the passed is different to it
		if (currentViewer == viewer) {
			return;
		}

		// currentViewer is null at first call
		if (currentViewer != null) {
			currentViewer.removeSelectionChangedListener(this);
		}

		currentViewer = viewer;
		currentViewer.getControl().setFocus();

		// register this as a listener on the viewer to update actions on
		// selection changes.
		currentViewer.addSelectionChangedListener(this);

		// set current viewer as actual selection provider for this page's site
		selectionProvider.setSelectionProviderDelegate(currentViewer);

		updateActions();

		// update current viewer as source of Thumbnail
		final ScalableFreeformRootEditPart rootEditPart = (ScalableFreeformRootEditPart) currentViewer
				.getRootEditPart();
		thumbnail.setViewport((Viewport) rootEditPart.getFigure());
		thumbnail.setSource(rootEditPart
				.getLayer(LayerConstants.PRINTABLE_LAYERS));
		thumbnail.setVisible(true);

		// update current viewer's ZoomManager for ZoomManagerDelegate
		zoomManagerDelegate.setCurrentZoomManager((ZoomManager) currentViewer
				.getProperty(ZoomManager.class.toString()));
	}

	/**
	 * Method to get the array of viewers if needed to implement advanced editor
	 * features.
	 * 
	 * <p>
	 * To set new contents for a viewer
	 * {@link #setViewersContents(int, EObject)} should be used!
	 * <p>
	 * 
	 * @return the viewers
	 * @see #setViewersContents(int, EObject)
	 */
	protected MultiViewerPageViewer[] getViewers() {
		return viewers;
	}

	/**
	 * This can be used to display messages in this page's status line.
	 * 
	 * @return The {@link IStatusLineManager} from the {@link IActionBars} of
	 *         this page's {@link IPageSite}
	 */
	public IStatusLineManager getStatusLineManager() {
		return getSite().getActionBars().getStatusLineManager();
	}

	/**
	 * This can be used to add actions to this page's tool bar.
	 * 
	 * @return The {@link IToolBarManager} from the {@link IActionBars} of this
	 *         page's {@link IPageSite}
	 */
	public IToolBarManager getToolBarManager() {
		return getSite().getActionBars().getToolBarManager();
	}

	/**
	 * If some command manipulates the model the actions are updated so that
	 * actions are not only updated if a selection change occurs.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.commands.CommandStackListener#commandStackChanged(java.util.EventObject)
	 */
	public void commandStackChanged(final EventObject event) {
		updateActions();
	}

	/**
	 * This local class extends {@link ScrollingGraphicalViewer} by a
	 * {@link MouseListener} that sets this viewer as the page's current viewer
	 * when a mouse click occurs on this viewer.
	 * 
	 * @author Tony Modica
	 */
	public class MultiViewerPageViewer extends ScrollingGraphicalViewer {

		/**
		 * A listener that notices when the mouse acts on this viewer.
		 */
		private final MouseListener mouseListener;

		public MultiViewerPageViewer() {
			super();
			mouseListener = new MouseAdapter() {
				@Override
				public void mouseDown(final MouseEvent e) {
					AbstractMultiViewerPage.this
							.setCurrentViewer(MultiViewerPageViewer.this);
				}
			};
		}

		/**
		 * @return The page that hosts this graphical viewer.
		 */
		public AbstractMultiViewerPage getHostPage() {
			return AbstractMultiViewerPage.this;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.gef.ui.parts.GraphicalViewerImpl#hookControl()
		 */
		@Override
		protected void hookControl() {
			super.hookControl();
			getControl().addMouseListener(mouseListener);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.gef.ui.parts.GraphicalViewerImpl#unhookControl()
		 */
		@Override
		protected void unhookControl() {
			getControl().removeMouseListener(mouseListener);
			super.unhookControl();
		}
	}

}
