package roneditor;

import java.io.ByteArrayInputStream;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.EventObject;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.Vector;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Plugin;
import org.eclipse.core.runtime.Preferences;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.impl.AdapterImpl;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.gef.ContextMenuProvider;
import org.eclipse.gef.DefaultEditDomain;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartFactory;
import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.KeyHandler;
import org.eclipse.gef.KeyStroke;
import org.eclipse.gef.TreeEditPart;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.gef.commands.CommandStackListener;
import org.eclipse.gef.ui.actions.ActionBarContributor;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.gef.ui.actions.DeleteAction;
import org.eclipse.gef.ui.actions.DirectEditAction;
import org.eclipse.gef.ui.actions.GEFActionConstants;
import org.eclipse.gef.ui.actions.PrintAction;
import org.eclipse.gef.ui.actions.RedoAction;
import org.eclipse.gef.ui.actions.SaveAction;
import org.eclipse.gef.ui.actions.SelectAllAction;
import org.eclipse.gef.ui.actions.UndoAction;
import org.eclipse.gef.ui.actions.UpdateAction;
import org.eclipse.gef.ui.parts.SelectionSynchronizer;
import org.eclipse.gef.ui.parts.TreeViewer;
import org.eclipse.gef.ui.properties.UndoablePropertySheetEntry;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Widget;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.IPerspectiveDescriptor;
import org.eclipse.ui.IPerspectiveFactory;
import org.eclipse.ui.ISelectionService;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.eclipse.ui.dialogs.SaveAsDialog;
import org.eclipse.ui.part.EditorPart;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.eclipse.ui.views.properties.IPropertySheetEntry;
import org.eclipse.ui.views.properties.PropertySheetPage;
import org.eclipse.ui.views.properties.PropertySheetSorter;
import org.osgi.framework.Bundle;

/**
 * <p>
 * This is a basic abstract implementation of an {@link EditorPart} with a
 * {@link TreeViewer}. It is designed to be used in an Eclipse plugin as it
 * demands a {@link AbstractUIPlugin} to be able to access some of its
 * {@link Bundle} information.
 * </p>
 * 
 * <p>
 * Prerequisites:
 * <ul>
 * <li> As extensions in the plugin.xml there have to be defined <b>exactly</b>
 * one of each:
 * <ul>
 * <li> "org.eclipse.ui.editors", must be pointing to the concrete subclass.
 * <li> "org.eclipse.ui.perspectives", must be pointing to a
 * {@link IPerspectiveFactory} that describes the perspective for this editor.
 * </ul>
 * </ul>
 * </p>
 * 
 * <p>
 * When instantiated (and in {@link #createPartControl(Composite)}) this class
 * does the following:
 * <ol>
 * <li> It creates a {@link DefaultEditDomain} and sets itself as the
 * {@link EditorPart}.
 * <li> Looks up the perspective defined for the plugin and opens it.
 * <li> A {@link TreeViewer} is created.
 * <li> Sets the TreeViewer in the EditDomain and listen to selection changes on
 * it.
 * <li> Installs a {@link SelectionSynchronizer} on the {@link TreeViewer}.
 * <li> {@link TreeViewer} is set as {@link ISelectionProvider} for this
 * {@link IEditorSite}.
 * <li> Sets the edit part factory, contents, and context menu for the
 * TreeViewer determined by abstract methods.
 * <li> Sets a standard {@link KeyHandler} for the {@link TreeViewer}.
 * </ol>
 * </p>
 * 
 * <p>
 * Additionally it provides the following features:
 * <ul>
 * <li> Errors and Exceptions should be logged for the plugin using
 * {@link #logError(String, Exception)}.
 * <li> These actions are created and registered by default:
 * <ul>
 * <li> {@link SaveAction},
 * <li> {@link UndoAction}, {@link RedoAction},
 * <li> {@link SelectAllAction}, {@link PrintAction},
 * <li> {@link DirectEditAction}, {@link DeleteAction}
 * </ul>
 * <li> The {@link CommandStack} of the {@link EditDomain} is watched. When
 * changed the actions of {@link #getStackActions()} are updated. <br>
 * By default {@link UndoAction} and {@link RedoAction} are updated then.
 * <li> The {@link TreeViewer} is watched for selection changes. When occuring
 * the actions of {@link #getSelectionActions()} are updated. <br>
 * By default {@link DirectEditAction} and {@link DeleteAction} are updated
 * then.
 * <li> When {@link #firePropertyChange(int)} occurs the actions of
 * {@link #getPropertyActions()} are updated. <br>
 * By default, this is done for {@link SaveAction}, as it is intended to react
 * on firing of {@link IEditorPart#PROP_DIRTY}.
 * <li> When the {@link CommandStack} is changed the editor checks its own dirty
 * state and fires {@link IEditorPart#PROP_DIRTY} if it changes.
 * <li> When closed, the editor closes the editor's perspective if no other
 * editor with the ID declared in plugin.xml is active.
 * <li> With {@link #createIDForModel(EObject)} the passed model will be
 * registered with an unique ID that is used with {@link AbstractPageBookView}s
 * to tell them with their secondary ID which model to show. These views make
 * use of {@link #getModelForID(String)} to resolve the ID to the model. <br>
 * It is expected that views get opened from within {@link EditPart}s via a
 * call
 * <code>activePage.showView(viewID, editor.createIDForModel(model), IWorkbenchPage.VIEW_ACTIVATE);</code>
 * where
 * <ul>
 * <li> viewID is the ID which this view has been registered with in the
 * plugin.xml.
 * <li> model is the model to be displayed.
 * <li> editor is this.
 * </ul>
 * <li> The editor will hide such a view automatically when its model has been
 * deleted (i.e. model.eResource() == <code>null</code>).
 * </ul>
 * <li> When being disposed the editor stores information (IDs) about the opened
 * views to the plugin's preferences to reopen them when the editor is started
 * again.
 * </p>
 * 
 * <p>
 * These methods have to be implemented by subclasses:
 * <ul>
 * <li> {@link #getActivator()}: The {@link AbstractUIPlugin} activator for
 * this eclipse plugin.
 * <li> {@link #createContextMenuProvider(TreeViewer)}: The
 * {@link ContextMenuProvider} for the {@link TreeViewer}.
 * <li> {@link #getModelRoot()}: The top model element to be set as contents of
 * the {@link TreeViewer}.
 * <li> {@link #createTreeEditPartFactory()}: A suitable
 * {@link EditPartFactory} for the model elements.
 * <li> {@link #save(IFile, IProgressMonitor)}: A method that handles saving
 * the model to a {@link IFile}.
 * </ul>
 * </p>
 * 
 * @author Tony Modica
 */
public abstract class AbstractTreeEditor extends EditorPart implements
		CommandStackListener, ISelectionChangedListener {

	/**
	 * The {@link EditDomain} of this editor.
	 */
	private final DefaultEditDomain editDomain;

	/**
	 * The {@link TreeViewer} in this editor.
	 */
	private TreeViewer treeViewer;

	/**
	 * The {@link ActionRegistry} containing the actions created by this editor.
	 */
	private ActionRegistry actionRegistry;

	/**
	 * The {@link SelectionSynchronizer} managing the selections for this editor
	 * and the views it can open.
	 */
	private SelectionSynchronizer synchronizer;

	/**
	 * The actions of this editor depending on the actual selection.
	 */
	private final List<IAction> selectionActions = new ArrayList<IAction>();

	/**
	 * The actions of this editor depending on the actual command stack.
	 */
	private final List<IAction> stackActions = new ArrayList<IAction>();

	/**
	 * The actions of this editor depending on property change notifications.
	 */
	private final List<IAction> propertyActions = new ArrayList<IAction>();

	/**
	 * The key handler of this editor.
	 */
	private KeyHandler keyHandler;

	/**
	 * A helper flag to decide if the save action should be enabled.
	 */
	private boolean previousDirtyState;

	/**
	 * A map to resolve unique ids to ecore models. This is used to allow views
	 * opened to show a specific model to resolve their secondary id to the
	 * model to be shown.
	 */
	private final Map<String, EObject> id2ModelMap = new HashMap<String, EObject>();

	/**
	 * A map reversing id2ModelMap to get possible previously generated ids for
	 * models rather that create new ones.
	 */
	private final Map<EObject, String> model2IdMap = new HashMap<EObject, String>();

	/**
	 * A map only used to be able to remove deletion listeners from the models
	 * when the editor is being closed.
	 */
	private final Map<EObject, Adapter> model2ListenerMap = new HashMap<EObject, Adapter>();

	/**
	 * The standard constructor creates a {@link DefaultEditDomain} and
	 * registers itself as a {@link CommandStackListener} on the domains
	 * {@link CommandStack}.
	 */
	protected AbstractTreeEditor() {
		editDomain = new DefaultEditDomain(this);
		getCommandStack().addCommandStackListener(this);
		previousDirtyState = false;
	}

	/**
	 * @return A lazily created {@link KeyHandler} for this editor.
	 */
	protected KeyHandler getKeyHandler() {
		if (keyHandler == null) {
			keyHandler = new KeyHandler();
			keyHandler
					.put(KeyStroke.getPressed(SWT.DEL, 127, 0),
							getActionRegistry().getAction(
									ActionFactory.DELETE.getId()));
			keyHandler.put(KeyStroke.getPressed(SWT.F2, 0), getActionRegistry()
					.getAction(GEFActionConstants.DIRECT_EDIT));
		}
		return keyHandler;
	}

	/**
	 * When the command stack changes, the actions interested in the command
	 * stack are updated. Additionally the editor checks if itself has changed
	 * its dirty state by comparing its <i>previous</i> dirty state with the
	 * actual command stack state.
	 * 
	 * @param event
	 *            the change event
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.commands.CommandStackListener#commandStackChanged(java.util.EventObject)
	 */
	public void commandStackChanged(final EventObject event) {
		if (isDirty() != previousDirtyState) {
			previousDirtyState = isDirty();
			firePropertyChange(IEditorPart.PROP_DIRTY);
		}
		updateActions(getStackActions());
	}

	/**
	 * Creates some standard GEF actions for this editor. Subclasses should
	 * override this method to create and register custom actions with the
	 * {@link ActionRegistry}.
	 */
	protected void createActions() {
		IAction action;

		action = new SaveAction(this);
		getActionRegistry().registerAction(action);
		getPropertyActions().add(action);

		action = new UndoAction(this);
		getActionRegistry().registerAction(action);
		getStackActions().add(action);

		action = new RedoAction(this);
		getActionRegistry().registerAction(action);
		getStackActions().add(action);

		action = new SelectAllAction(this);
		getActionRegistry().registerAction(action);

		action = new PrintAction(this);
		getActionRegistry().registerAction(action);

		action = new DirectEditAction((IWorkbenchPart) this);
		getActionRegistry().registerAction(action);
		getSelectionActions().add(action);

		action = new DeleteAction((IWorkbenchPart) this);
		getActionRegistry().registerAction(action);
		getSelectionActions().add(action);
	}

	/**
	 * Creates the GraphicalViewer on the specified <code>Composite</code>.
	 * 
	 * @param parent
	 *            the parent composite
	 */
	protected void createTreeViewer(final Composite parent) {
		getTreeViewer().createControl(parent);
		getTreeViewer().getControl().setBackground(
				parent.getDisplay().getSystemColor(SWT.COLOR_LIST_BACKGROUND));
		getTreeViewer().getControl().setForeground(
				parent.getDisplay().getSystemColor(SWT.COLOR_LIST_FOREGROUND));

		// set the contents, edit part factory and context menu for the
		// TreeViewer
		getTreeViewer().setEditPartFactory(createTreeEditPartFactory());
		getTreeViewer().setContents(getModelRoot());
		final ContextMenuProvider provider = createContextMenuProvider(getTreeViewer());
		getTreeViewer().setContextMenu(provider);

		// configure the key handler
		getTreeViewer().setKeyHandler(getKeyHandler());
	}

	/**
	 * Realizes the editor by creating its {@link Control}.
	 * <P>
	 * WARNING: This method may or may not be called by the workbench prior to
	 * {@link #dispose()}.
	 * 
	 * @param parent
	 *            the parent composite
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IWorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	public void createPartControl(final Composite parent) {
		createTreeViewer(parent);

		// The viewer is added to the SelectionSynchronizer, which can be used
		// to keep 2 or more EditPartViewers in sync. The viewer is also
		// registered as the ISelectionProvider for the Editor's PartSite.
		getSelectionSynchronizer().addViewer(getTreeViewer());
		getSite().setSelectionProvider(getTreeViewer());

		// hook TreeViewer in EditDomain and listen to selection changes
		getEditDomain().addViewer(getTreeViewer());
		getTreeViewer().addSelectionChangedListener(this);

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IWorkbenchPart#dispose()
	 */
	@Override
	public void dispose() {
		getCommandStack().removeCommandStackListener(this);
		getTreeViewer().removeSelectionChangedListener(this);
		getEditDomain().setActiveTool(null);
		getActionRegistry().dispose();

		// clean up model registry and store IDs of open views
		storeAndCleanModelRegistry();

		// close perspective if the last editor instance is being closed
		final IWorkbenchPage page = getSite().getPage();
		if (!isAnotherEditorActive()) {
			final IPerspectiveDescriptor perspective = page.getPerspective();
			if (perspective.getId().equals(
					getUniqueExtensionAttributeValue(
							"org.eclipse.ui.perspectives", "id"))) {
				page.closePerspective(perspective, true, false);
			}
		}
		super.dispose();
	}

	/**
	 * A helper method that looks for other active instances of this editor in
	 * the site's page.
	 * 
	 * @return <code>true</code> if another instance of this editor is active
	 */
	protected boolean isAnotherEditorActive() {
		final IEditorReference[] editorRefs = getSite().getPage()
				.getEditorReferences();
		final String editorID = getUniqueExtensionAttributeValue(
				"org.eclipse.ui.editors", "id");
		for (final IEditorReference editorRef : editorRefs) {
			if (editorRef.getId().equals(editorID)) {
				return true;
			}
		}
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.WorkbenchPart#firePropertyChange(int)
	 */
	@Override
	protected void firePropertyChange(final int property) {
		super.firePropertyChange(property);
		updateActions(getPropertyActions());
	}

	/**
	 * Lazily creates, initializes and returns the action registry.
	 * 
	 * @return the action registry
	 */
	protected ActionRegistry getActionRegistry() {
		if (actionRegistry == null) {
			actionRegistry = new ActionRegistry();
			initializeActionRegistry();
		}
		return actionRegistry;
	}

	/**
	 * <P>
	 * <EM>IMPORTANT</EM> certain requests, such as the property sheet, may be
	 * made before or after {@link #createPartControl(Composite)} is called. The
	 * order is unspecified by the Workbench.
	 * 
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	 */
	@SuppressWarnings("unchecked")
	@Override
	public Object getAdapter(final Class type) {
		if (type == org.eclipse.ui.views.properties.IPropertySheetPage.class) {
			/*
			 * The PropertySheetPage is extended so that a PropertySheetSorter
			 * that does nothing (compares everything as equal) is being set.
			 * This way the properties will be displayed in the order they are
			 * returned by the property sources instead of being sorted
			 * alphabetically.
			 */
			final PropertySheetPage page = new PropertySheetPage() {
				/*
				 * (non-Javadoc)
				 * 
				 * @see org.eclipse.ui.views.properties.PropertySheetPage#createControl(org.eclipse.swt.widgets.Composite)
				 */
				@Override
				public void createControl(Composite parent) {
					setSorter(new PropertySheetSorter() {
						@Override
						public int compare(IPropertySheetEntry entryA,
								IPropertySheetEntry entryB) {
							return 0;
						}
					});
					super.createControl(parent);
				}
			};
			// set the PropertySheetPage to use our command stack
			page
					.setRootEntry(new UndoablePropertySheetEntry(
							getCommandStack()));
			return page;
		}
		if (type == TreeViewer.class) {
			return getTreeViewer();
		}
		if (type == CommandStack.class) {
			return getCommandStack();
		}
		if (type == ActionRegistry.class) {
			return getActionRegistry();
		}
		if (type == EditPart.class && getTreeViewer() != null) {
			return getTreeViewer().getRootEditPart();
		}
		if (type == Widget.class && getTreeViewer() != null) {
			return ((TreeEditPart) getTreeViewer().getRootEditPart())
					.getWidget();
		}
		if (type == KeyHandler.class) {
			return getKeyHandler();
		}
		if (type == SelectionSynchronizer.class) {
			return getSelectionSynchronizer();
		}
		if (type == AbstractUIPlugin.class || type == Plugin.class) {
			return getActivator();
		}
		if (type == IResource.class) {
			return ((IFileEditorInput) getEditorInput()).getFile();
		}
		return super.getAdapter(type);
	}

	protected CommandStack getCommandStack() {
		return getEditDomain().getCommandStack();
	}

	protected DefaultEditDomain getEditDomain() {
		return editDomain;
	}

	/**
	 * @return A lazily created {@link EditorTreeViewer} that allows to access
	 *         the editor.
	 */
	protected TreeViewer getTreeViewer() {
		if (treeViewer == null) {
			treeViewer = new EditorTreeViewer();
		}
		return treeViewer;
	}

	/**
	 * Returns the list of {@link IAction IActions} dependent on property
	 * changes in the Editor. These actions should implement the
	 * {@link UpdateAction} interface so that they can be updated in response to
	 * property changes. An example is the "Save" action.
	 * 
	 * @return the list of property-dependent actions
	 */
	protected List<IAction> getPropertyActions() {
		return propertyActions;
	}

	/**
	 * Returns the list of {@link IAction} that depend on changes in the
	 * workbench's {@link ISelectionService}. These actions can be updated in
	 * response to selection changes. An example is the Delete action.
	 * 
	 * @return the list of selection-dependent actions
	 */
	protected List<IAction> getSelectionActions() {
		return selectionActions;
	}

	/**
	 * Returns the selection synchronizer object. The synchronizer can be used
	 * to sync the selection of 2 or more EditPartViewers.
	 * 
	 * @return A lazily created {@link SelectionSynchronizer}
	 */
	protected SelectionSynchronizer getSelectionSynchronizer() {
		if (synchronizer == null) {
			synchronizer = new SelectionSynchronizer() {
				@Override
				protected EditPart convert(final EditPartViewer viewer,
						final EditPart part) {
					// We assume that we have an EMF model
					final Object model = part.getModel();
					if (model instanceof EObject) {
						return convertEObject(viewer, (EObject) model);
					}
					// else look if the (possible existing) parent has a EObject
					// as model
					if (part.getParent() != null) {
						return convert(viewer, part.getParent());
					}
					return null;
				}

				/**
				 * @param viewer
				 *            the viewer being mapped to
				 * @param model
				 *            the ecore model element of a part from another
				 *            viewer
				 * @return An edit part of an (in)direct parent of the model in
				 *         the viewer or <code>null</code>
				 */
				private EditPart convertEObject(final EditPartViewer viewer,
						final EObject model) {
					final Object temp = viewer.getEditPartRegistry().get(model);
					if (temp != null) {
						return (EditPart) temp;
					}
					if (model.eContainer() != null) {
						return convertEObject(viewer, model.eContainer());
					}
					return null;
				}
			};
		}
		return synchronizer;
	}

	/**
	 * Returns the list of {@link IAction}s dependent on the CommmandStack's
	 * state. These actions should implement the {@link UpdateAction} interface
	 * so that they can be updated in response to command stack changes. An
	 * example is the "undo" action.
	 * 
	 * @return the list of stack-dependent actions
	 */
	protected List<IAction> getStackActions() {
		return stackActions;
	}

	/**
	 * Sets the site and input for this editor and opens a unique perspective
	 * defined in plugin.xml. Subclasses may extend this method, but should
	 * always call <code>super.init(site, input)</code>.
	 * 
	 * @see org.eclipse.ui.IEditorPart#init(org.eclipse.ui.IEditorSite,
	 *      org.eclipse.ui.IEditorInput)
	 */
	@Override
	public void init(final IEditorSite site, final IEditorInput input) {
		setSite(site);
		setInput(input);

		// Using the active page of the workbench is a bugfix for that the page
		// layout is corrupted if the perspective is set on getSite().getPage()
		final IWorkbenchPage page = PlatformUI.getWorkbench()
				.getActiveWorkbenchWindow().getActivePage();

		if (null != page) {
			// open editor perspective
			final String perspectiveID = getUniqueExtensionAttributeValue(
					"org.eclipse.ui.perspectives", "id");
			if (null != perspectiveID) {
				final IPerspectiveDescriptor editorPerspective = PlatformUI
						.getWorkbench().getPerspectiveRegistry()
						.findPerspectiveWithId(perspectiveID);
				if (page.getPerspective() != editorPerspective) {
					page.setPerspective(editorPerspective);
				}
			}
		}
		restoreModelRegistry();
	}

	/**
	 * Initializes the action registry. This registry may be used by {@link
	 * ActionBarContributor ActionBarContributors} and/or
	 * {@link ContextMenuProvider ContextMenuProviders}.
	 * <p>
	 * This method is being called the first time {@link #getActionRegistry()}
	 * gets called.
	 * </p>
	 */
	protected void initializeActionRegistry() {
		createActions();
		updateActions(getPropertyActions());
		updateActions(getStackActions());
	}

	/**
	 * To be implemented by concrete subclasses! <br>
	 * To access the active page
	 * PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage()
	 * should be used.
	 * 
	 * @return The Activator instance of this plug-in. Needed to access the
	 *         plugin's {@link Bundle} and {@link IExtension}s.
	 */
	abstract protected AbstractUIPlugin getActivator();

	/**
	 * @param exPointID
	 *            The extension to be looked at.
	 * @param attribID
	 *            The attribute whose value is requested.
	 * @return The value of the specified attribute.
	 * @throws CoreException
	 *             This is thrown if no (unique) value can be determined.
	 */
	protected String getUniqueExtensionAttributeValue(final String exPointID,
			final String attribID) {
		final Bundle bundle = getActivator().getBundle();
		final String pluginName = bundle.getSymbolicName();
		final IExtension[] extensions = Platform.getExtensionRegistry()
				.getExtensions(pluginName);
		String attrValue = null;
		for (final IExtension extension : extensions) {
			// look for the given extension id
			if (extension.getExtensionPointUniqueIdentifier().equals(exPointID)) {
				final IConfigurationElement[] confElems = extension
						.getConfigurationElements();
				if (confElems.length != 1) {
					logError("The Plugin " + pluginName
							+ " does not specify a unique extension with ID "
							+ exPointID, null);
				}
				attrValue = confElems[0].getAttribute(attribID);
				if (null == attrValue) {
					logError("The extension " + exPointID + " of the plugin "
							+ pluginName
							+ " does not specify a unique attribute with ID "
							+ attribID, null);
				}
			}
		}
		return attrValue;
	}

	/**
	 * @param exPointID
	 *            The extension to be looked at.
	 * @param attribID
	 *            The attribute whose value is requested.
	 * @return The values of the specified attribute in a {@link Vector}.
	 * @throws CoreException
	 *             This is thrown if no (unique) value can be determined.
	 */
	protected Vector<String> getExtensionAttributeValues(
			final String exPointID, final String attribID) {
		final Bundle bundle = getActivator().getBundle();
		final String pluginName = bundle.getSymbolicName();
		final IExtension[] extensions = Platform.getExtensionRegistry()
				.getExtensions(pluginName);
		final Vector<String> attrValues = new Vector<String>();
		for (final IExtension extension : extensions) {
			// look for the given extension id
			if (extension.getExtensionPointUniqueIdentifier().equals(exPointID)) {
				final IConfigurationElement[] confElems = extension
						.getConfigurationElements();
				for (final IConfigurationElement confElem : confElems) {
					attrValues.add(confElem.getAttribute(attribID));
				}
			}
		}
		return attrValues;
	}

	/**
	 * @param message
	 *            The message for the {@link IStatus} to be logged for the
	 *            plugin determined by {@link #getActivator()}.
	 * @param e
	 *            An optional Exception to be reported.
	 */
	public void logError(final String message, final Exception e) {
		final String pluginName = getActivator().getBundle().getSymbolicName();
		final IStatus status = new Status(IStatus.ERROR, pluginName,
				IStatus.OK, message, e);
		getActivator().getLog().log(status);
	}

	/**
	 * @param viewer
	 *            The {@link TreeViewer} for that this
	 *            {@link ContextMenuProvider} constructs a menu.
	 */
	abstract protected ContextMenuProvider createContextMenuProvider(
			TreeViewer viewer);

	/**
	 * @return The model element object that is the root of the model.
	 */
	abstract protected EObject getModelRoot();

	/**
	 * @return The {@link EditPartFactory} for this GEF {@link TreeViewer}.
	 */
	abstract protected EditPartFactory createTreeEditPartFactory();

	/**
	 * Returns <code>true</code> if the command stack is dirty
	 * 
	 * @see org.eclipse.ui.ISaveablePart#isDirty()
	 */
	@Override
	public boolean isDirty() {
		return getCommandStack().isDirty();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IWorkbenchPart#setFocus()
	 */
	@Override
	public void setFocus() {
		getTreeViewer().getControl().setFocus();
		updateActions(getSelectionActions());
	}

	/**
	 * A convenience method for updating a set of actions defined by the given
	 * List of actions.
	 * 
	 * @param action
	 *            The list of actions to update.
	 */
	protected void updateActions(final List<IAction> actions) {
		for (final Object action : actions) {
			if (action instanceof UpdateAction) {
				((UpdateAction) action).update();
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.EditorPart#isSaveAsAllowed()
	 */
	@Override
	public boolean isSaveAsAllowed() {
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.EditorPart#doSave(org.eclipse.core.runtime.IProgressMonitor)
	 */
	@Override
	public void doSave(final IProgressMonitor monitor) {
		try {
			final IFile file = ((IFileEditorInput) getEditorInput()).getFile();
			if (!file.exists()) {
				file.create(new ByteArrayInputStream(new byte[0]), true,
						new SubProgressMonitor(monitor, 5));
			}
			save(file, monitor);
			getCommandStack().markSaveLocation();
		} catch (final CoreException e) {
			ErrorDialog.openError(getSite().getShell(), "Error during save",
					"The current model could not be saved!", e.getStatus());
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.EditorPart#doSaveAs()
	 */
	@Override
	public void doSaveAs() {
		final SaveAsDialog dialog = new SaveAsDialog(getSite().getShell());
		dialog.setTitle("Save as...");
		dialog.setOriginalFile(((IFileEditorInput) getEditorInput()).getFile());
		dialog.open();
		final IPath path = dialog.getResult();
		if (dialog.getReturnCode() == Window.CANCEL || path == null) {
			return;
		}
		final WorkspaceModifyOperation op = new WorkspaceModifyOperation() {
			@SuppressWarnings("synthetic-access")
			@Override
			public void execute(final IProgressMonitor monitor)
					throws CoreException {
				try {
					IFile newFile = ResourcesPlugin.getWorkspace().getRoot()
							.getFile(path);
					save(newFile, monitor);
					getCommandStack().markSaveLocation();
					setInput(new FileEditorInput(newFile));
				} catch (final CoreException e) {
					ErrorDialog.openError(getSite().getShell(),
							"Error during save",
							"The current model could not be saved!", e
									.getStatus());
					throw e;
				}
			}
		};

		try {
			new ProgressMonitorDialog(getSite().getWorkbenchWindow().getShell())
					.run(false, true, op);
		} catch (final InvocationTargetException e) {
			e.printStackTrace();
		} catch (final InterruptedException e) {
			e.printStackTrace();
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.ISelectionChangedListener#selectionChanged(org.eclipse.jface.viewers.SelectionChangedEvent)
	 */
	public void selectionChanged(final SelectionChangedEvent event) {
		updateActions(getSelectionActions());
	}

	/**
	 * This method saves the model to a file using some facility like EMF
	 * {@link Resource}s.
	 * 
	 * @param file
	 *            The {@link IFile} to save the model to.
	 * @param progressMonitor
	 *            A progress monitor that could be used to show the saving
	 *            status.
	 * @throws CoreException
	 *             This exception indicates that something went wrong during
	 *             saving.
	 */
	abstract protected void save(final IFile file,
			final IProgressMonitor progressMonitor) throws CoreException;

	/**
	 * Opens a view on this side's page displaying a model. An unique ID for the
	 * passed model is being created and set as secondary ID of the new view.
	 * The model for this ID can be resolved via {@link #getModelForID(String)}.
	 * 
	 * @param viewId
	 *            The primary ID of the view to be opened. Should be
	 *            correspondent to an org.eclipse.ui.views entry in plugin.xml
	 * @param model
	 *            The model to be shown in the view
	 */
	public void showView(final String viewId, final EObject model) {
		try {
			getSite().getPage().showView(viewId, createIDForModel(model),
					IWorkbenchPage.VIEW_ACTIVATE);
		} catch (final PartInitException e) {
			final String message = "View with ID " + viewId
					+ " could not be initialized!";
			logError(message, e);
		}
	}

	/**
	 * Method to resolve an ID to a ECore model element. This ID has to be
	 * created explicitly before.
	 * 
	 * @param id
	 *            The id that identifies the wanted model
	 * @return The model that has been registered with the id
	 * 
	 * @see #createIDForModel(EObject)
	 */
	public EObject getModelForID(final String id) {
		return id2ModelMap.get(id);
	}

	/**
	 * Generates an unique ID for this model and registers them with each other
	 * in two maps. The ID is to be used as secondary ID to tell views what
	 * model they shall display. If the model has already been registered
	 * before, the previous generated ID is returned rather than a newly created
	 * one. <br>
	 * 
	 * Additionally an {@link Adapter} is added to the model. The adapter will
	 * close the views that have been opened with this ID as secondary ID
	 * automatically when deletion of the model is being notified.
	 * 
	 * @param model
	 *            The model to get an ID for
	 * @return A unique ID for the model
	 */
	public String createIDForModel(final EObject model) {
		return createIDForModel(model, null);
	}

	/**
	 * Generates an unique ID for this model and registers them with each other
	 * in two maps. The ID is to be used as secondary ID to tell views what
	 * model they shall display. If the model has already been registered
	 * before, the previous generated ID is returned rather than a newly created
	 * one. <br>
	 * 
	 * Additionally an {@link Adapter} is added to the model. The adapter will
	 * close the views that have been opened with this ID as secondary ID
	 * automatically when deletion of the model is being notified. *
	 * 
	 * @param model
	 *            The model to get an ID for
	 * @param forceID
	 *            May be used to force a particular ID for a model. If
	 *            <code>null</code>, a new ID will be generated.
	 * @return A unique ID for the model
	 */
	private String createIDForModel(final EObject model, String forceID) {
		if (model2IdMap.containsKey(model)) {
			return model2IdMap.get(model);
		}

		// listen to model for deletion
		final Adapter listener = new AdapterImpl() {
			/*
			 * (non-Javadoc)
			 * 
			 * @see org.eclipse.emf.common.notify.impl.AdapterImpl#notifyChanged(org.eclipse.emf.common.notify.Notification)
			 */
			@Override
			public void notifyChanged(Notification msg) {
				// Test if model has been deleted from the resource
				// Maybe the notification could tell somehow?
				if (null == model.eResource()) {
					// close views showing deleted models
					closeViewsShowing(model);
				}
			}
		};

		// add and register adapter
		model.eAdapters().add(listener);
		model2ListenerMap.put(model, listener);

		// generate new ID if not forced
		if (null == forceID) {
			forceID = EcoreUtil.generateUUID();
		}

		// check for uniqueness of ID, should never occur
		Assert.isTrue(!id2ModelMap.containsKey(forceID),
				"Generated or forced ID has already been assigned to a model!");

		// register crosswise
		model2IdMap.put(model, forceID);
		id2ModelMap.put(forceID, model);
		return forceID;
	}

	/**
	 * Closes alls views in the site's page whose secondary ID correspond to the
	 * id the passed model has been registered with.
	 * 
	 * @param model
	 *            The model
	 * @return The (primary) id of the last closed view showing the model if
	 *         any, else <code>null</code>
	 */
	String closeViewsShowing(final EObject model) {
		final IWorkbenchPage page = getSite().getPage();
		final String modelID = model2IdMap.get(model);
		final IViewReference[] viewRefs = page.getViewReferences();
		String foundViewID = null;
		for (final IViewReference viewRef : viewRefs) {
			if (modelID.equals(viewRef.getSecondaryId())) {
				page.hideView(viewRef.getView(true));
				foundViewID = viewRef.getId();
			}
		}
		return foundViewID;
	}

	/**
	 * Unregisters each registered model and closes the views that show them.
	 * The IDs and their correspondent model URIs for the currently open views
	 * are stored to the plugin's preferences. <br>
	 * To be called, when the editor is being closed.
	 * 
	 * @see #restoreModelRegistry()
	 */
	private void storeAndCleanModelRegistry() {
		final Preferences prefs = getActivator().getPluginPreferences();
		final StringBuffer modelIDs = new StringBuffer();

		for (final EObject model : new Vector<EObject>(model2IdMap.keySet())) {
			// remove listener
			model.eAdapters().remove(model2ListenerMap.get(model));

			// secondary IDs of views that are being closed should be stored to
			// preferences
			final String viewID = closeViewsShowing(model);
			if (null != viewID) {
				final String uriFragment = model.eResource().getURIFragment(
						model);
				modelIDs.append(uriFragment);
				modelIDs.append("###");
				final String id = model2IdMap.get(model);
				prefs.setValue(uriFragment, id);
				prefs.setValue(id, viewID);
			}

			// remove model and its ID from maps
			id2ModelMap.remove(model2IdMap.get(model));
			model2IdMap.remove(model);
			model2ListenerMap.remove(model);
		}

		// store URI of the model root's resource and the concatenated string of
		// model IDs whose views have been closed
		final String resURI = getModelRoot().eResource().getURI().toString();
		prefs.setValue("resourceURI", resURI);
		prefs.setValue(resURI, modelIDs.toString());

		// just to be sure
		if (!id2ModelMap.isEmpty() || !model2IdMap.isEmpty()
				|| !model2ListenerMap.isEmpty()) {
			logError(
					"Deregistering completely IDs and models failed in AbstractTreeEditor!",
					null);
		}
	}

	/**
	 * Restores the model-id-mappings from the plugin's preferences if the
	 * editor is still working on the same file. Tries to reopen the views that
	 * were opened when the editor was closed. The preferences are cleaned up
	 * afterwards.
	 * 
	 * @see #storeAndCleanModelRegistry()
	 */
	private void restoreModelRegistry() {
		final Preferences prefs = getActivator().getPluginPreferences();
		final String resourceURI = prefs.getString("resourceURI");

		// test if the model opened by this editor instance is the one whose
		// element IDs have been stored to the preferences
		final boolean doRestore = resourceURI.equals(getModelRoot().eResource()
				.getURI().toString());

		// prepare
		final Resource res = getModelRoot().eResource();
		final String modelIDs = prefs.getString(resourceURI);
		final StringTokenizer uriTokenizer = new StringTokenizer(modelIDs,
				"###");
		while (uriTokenizer.hasMoreTokens()) {
			final String uri = uriTokenizer.nextToken();
			if (doRestore) {
				final String id = prefs.getString(uri);
				final String viewID = prefs.getString(id);
				try {
					getSite().getPage().showView(viewID,
							createIDForModel(res.getEObject(uri), id),
							IWorkbenchPage.VIEW_ACTIVATE);
				} catch (final PartInitException e) {
					// just try to open the views, may not work but is not
					// important
				}
			}
			// delete from preferences
			prefs.setToDefault(uri);
		}

		// clean up the rest
		prefs.setToDefault("resourceURI");
		prefs.setToDefault(resourceURI);
	}

	/**
	 * This special TreeViewer is needed to allow the {@link EditPart}s to
	 * access the editor e.g. to generate IDs for models.
	 */
	public class EditorTreeViewer extends TreeViewer {
		/**
		 * @return The editor that hosts this tree viewer.
		 */
		public AbstractTreeEditor getEditor() {
			return AbstractTreeEditor.this;
		}
	}

}
