package roneditor.objectnetview.edit.parts;

import java.util.List;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.NodeEditPart;
import org.eclipse.ui.views.properties.IPropertySource;

import roneditor.objectnetview.edit.policies.ONNodeComponentEditPolicy;
import roneditor.objectnetview.edit.policies.ONNodeEditPolicy;
import roneditor.objectnetview.figures.ONPlaceFigure;
import roneditor.objectnetview.model.ONPlacePropertySource;
import ronmodel.ONPlace;
import ronmodel.RonmodelPackage;

/**
 * EditPart for a {@link ONPlace}.
 */
public class ONPlaceEditPart extends AbstractONNodeEditPart implements
		NodeEditPart, Adapter {
	private IPropertySource propertySource = null;

	private Notifier target;

	/**
	 * Creates a new {@link ONPlaceEditPart} instance.
	 * 
	 * @param onPlace
	 */
	public ONPlaceEditPart(ONPlace onPlace) {
		setModel(onPlace);
	}
	/**
	 * 
	 * @return the {@link ONPlace onPlace}
	 */
	public ONPlace getONPlace() {
		return (ONPlace) getModel();
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getModelSourceConnections()
	 */
	protected List getModelSourceConnections() {
		return getONPlace().getIncoming();
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getModelTargetConnections()
	 */
	protected List getModelTargetConnections() {
		return getONPlace().getOutgoing();
	}

	/**
	 * Returns the Figure of this, as a {@link ONPlaceFigure onPlace type figure}.
	 * 
	 * @return Figure as a {@link ONPlaceFigure}.
	 */
	protected ONPlaceFigure getPlaceFigure() {
		return (ONPlaceFigure) getFigure();
	}

	/**
	 * @see org.eclipse.emf.common.notify.Adapter#notifyChanged(org.eclipse.emf.common.notify.Notification)
	 */
	public void notifyChanged(Notification notification) {
		int featureId = notification.getFeatureID(RonmodelPackage.class);
		switch (featureId) {
		case RonmodelPackage.ON_NODE__OUTGOING:
			refreshTargetConnections();
			break;
		case RonmodelPackage.ON_NODE__INCOMING:
			refreshSourceConnections();
			break;
		default:
			refreshVisuals();
			break;
		}
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		// install the edit policy to handle connection creation
		installEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE,
				new ONNodeEditPolicy());
		// install the edit policy to handle deletion
		installEditPolicy(EditPolicy.COMPONENT_ROLE,
				new ONNodeComponentEditPolicy());
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 */
	protected void refreshVisuals() {
		getPlaceFigure().setName(getONPlace().getName());
		getPlaceFigure().setTokens(getONPlace().getTokens());
		getPlaceFigure().setShowingMappingID(
				!getONPlace().getOutgoingMappings().isEmpty()
						|| !getONPlace().getIncomingMappings().isEmpty());
		getPlaceFigure().setMappingID(getONPlace().getMappingID());
		Point loc = new Point(getONPlace().getX(), getONPlace().getY());
		Dimension size = new Dimension(-1, -1);
		Rectangle r = new Rectangle(loc, size);
		((GraphicalEditPart) getParent()).setLayoutConstraint(this,
				getFigure(), r);
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#activate()
	 */
	@SuppressWarnings("unchecked")
	public void activate() {
		if (isActive())
			return;

		// start listening for changes in the model

		getONPlace().eAdapters().add(this);

		super.activate();
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#deactivate()
	 */
	public void deactivate() {
		if (!isActive())
			return;

		// stop listening for changes in the model
		getONPlace().eAdapters().remove(this);

		super.deactivate();
	}

	/**
	 * @see org.eclipse.emf.common.notify.Adapter#getTarget()
	 */
	public Notifier getTarget() {
		return target;
	}

	/** 
	 * @see org.eclipse.emf.common.notify.Adapter#isAdapterForType(java.lang.Object)
	 */
	public boolean isAdapterForType(Object type) {
		return type.equals(getModel().getClass());
	}

	/** 
	 * @see org.eclipse.emf.common.notify.Adapter#setTarget(org.eclipse.emf.common.notify.Notifier)
	 */
	public void setTarget(Notifier newTarget) {
		target = newTarget;
	}

	/**
	 * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	 */
	public Object getAdapter(Class key) {
		if (IPropertySource.class == key) {
			return getPropertySource();
		}
		return super.getAdapter(key);
	}

	/** 
	 * @see com.ibm.itso.sal330r.gefdemo.edit.NetworkElementEditPart#getPropertySource()
	 */
	protected IPropertySource getPropertySource() {
		if (propertySource == null) {
			propertySource = new ONPlacePropertySource(getONPlace());
		}
		return propertySource;

	}

	/** 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	protected IFigure createFigure() {
		ONPlaceFigure figure = new ONPlaceFigure();

		return figure;
	}
}
