package roneditor.ronview.commands;

import java.util.Iterator;
import java.util.Vector;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.commands.Command;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PlatformUI;

import roneditor.objectnetview.ObjectNetPageBookView;
import roneditor.ruleview.RulePageBookView;
import ronmodel.HLArc;
import ronmodel.HLNode;
import ronmodel.HLPlace;
import ronmodel.ObjectNet;
import ronmodel.RON;
import ronmodel.Rule;

/**
 * Command for removing a node from its parent RON. This is undoable.
 */
public class DeleteHLNodeCommand extends Command {

	private static final String DeleteCommand_Label = "Delete high-level node";

	final private HLNode node;

	final private Vector<EObject> incomingHLArcs = new Vector<EObject>();

	final private Vector<EObject> outgoingHLArcs = new Vector<EObject>();

	private RON parent;

	public DeleteHLNodeCommand(final HLNode node) {
		super(DeleteHLNodeCommand.DeleteCommand_Label);
		this.node = node;
	}

	/**
	 * Delete all arcs which are connected with the transition.
	 * 
	 * @param hlNode
	 */
	private void deleteArcs(HLNode hlNode) {
		incomingHLArcs.clear();
		outgoingHLArcs.clear();
		for (Iterator it = hlNode.getIncoming().iterator(); it.hasNext();) {
			HLArc hlArc = (HLArc) it.next();
			HLNode source = hlArc.getSource();
			incomingHLArcs.add(hlArc);
			incomingHLArcs.add(source);
		}
		for (int i = 0; i < incomingHLArcs.size(); i = i + 2) {
			HLArc hlArc = (HLArc) incomingHLArcs.get(i);
			hlArc.setSource(null);
			hlArc.setTarget(null);
			hlArc.setRon(null);
		}

		for (Iterator it = hlNode.getOutgoing().iterator(); it.hasNext();) {
			HLArc hlArc = (HLArc) it.next();
			HLNode target = hlArc.getTarget();
			outgoingHLArcs.add(hlArc);
			outgoingHLArcs.add(target);
		}
		for (int i = 0; i < outgoingHLArcs.size(); i = i + 2) {
			HLArc hlArc = (HLArc) outgoingHLArcs.get(i);
			hlArc.setSource(null);
			hlArc.setTarget(null);
			hlArc.setRon(null);
		}
	}

	/**
	 * Executes the command by first hiding all views which display the element
	 * that will be removed. Then the child gets removed from its parent. The
	 * container is stored for undoing purposes.
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	@Override
	public void execute() {
		// Alle Views dieses Editors mssen geschlossen werden, da die URIs der
		// EObjects sich ndern und nicht mehr zu den Views passen.

		final IWorkbenchPage activePage = PlatformUI.getWorkbench()
				.getActiveWorkbenchWindow().getActivePage();
		final IViewReference[] viewRefs = activePage.getViewReferences();
		if (node instanceof HLPlace) {
			HLPlace place = (HLPlace) node;
			for (final IViewReference element : viewRefs) {
				if (element.getId().equals(
						"roneditor.objectnetview.ObjectNetPageBookView")) {
					ObjectNetPageBookView objectNetView = (ObjectNetPageBookView) element
							.getView(true);
					if (objectNetView.getModel() instanceof ObjectNet) {
						ObjectNet objectNet = (ObjectNet) objectNetView
								.getModel();
						if (objectNet.getPlace() == place) {
							activePage.hideView(element.getView(true));
						}
					}
				} else if (element.getId().equals(
						"roneditor.ruleview.RulePageBookView")) {
					RulePageBookView ruleView = (RulePageBookView) element
							.getView(true);
					if (ruleView.getModel() instanceof Rule) {
						Rule rule = (Rule) ruleView.getModel();
						if (rule.getPlace() == place) {
							activePage.hideView(element.getView(true));
						}
					}
				}
			}
		}

		// Jetzt kann das Element aus dem Container gelscht werden
		deleteArcs(node);
		parent = node.getRon();
		node.setRon(null);
	}

	/**
	 * Undoes the command by readding the previously removed object to its
	 * former container that {@link #execute()} has saved.
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	@Override
	public void undo() {

		for (int i = 0; i < incomingHLArcs.size(); i = i + 2) {
			HLArc hlArc = (HLArc) incomingHLArcs.get(i);
			HLNode source = (HLNode) incomingHLArcs.get(i + 1);
			hlArc.setSource(source);
			hlArc.setTarget(node);
			hlArc.setRon(parent);
		}
		for (int i = 0; i < outgoingHLArcs.size(); i = i + 2) {
			HLArc hlArc = (HLArc) outgoingHLArcs.get(i);
			HLNode target = (HLNode) outgoingHLArcs.get(i + 1);
			hlArc.setSource(node);
			hlArc.setTarget(target);
			hlArc.setRon(parent);
		}
		node.setRon(parent);
	}

	/**
	 * Redoes the command by removing the object from its container again.
	 */
	@Override
	public void redo() {
		execute();
	}
}
