package roneditor.ronview.edit.parts;

import java.util.List;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.NodeEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.ui.views.properties.IPropertySource;

import roneditor.actions.CreateTokenAction;
import roneditor.model.RONModelManager;
import roneditor.ronview.commands.CreateTokenCommand;
import roneditor.ronview.edit.policies.HLNodeComponentEditPolicy;
import roneditor.ronview.edit.policies.HLNodeEditPolicy;
import roneditor.ronview.figures.HLPlaceFigure;
import roneditor.ronview.model.HLPlacePropertySource;
import ronmodel.HLPlace;
import ronmodel.HLPlaceType;
import ronmodel.LHS;
import ronmodel.NAC;
import ronmodel.ObjectNet;
import ronmodel.RHS;
import ronmodel.RonmodelPackage;
import ronmodel.Rule;

/**
 * EditPart for a {@link HLPlace}.
 */
public class HLPlaceEditPart extends AbstractHLNodeEditPart implements
		NodeEditPart, Adapter {
	private IPropertySource propertySource = null;

	private Notifier target;

	/**
	 * Creates a new {@link HLPlaceEditPart} instance.
	 * 
	 * @param hlPlace
	 */
	public HLPlaceEditPart(HLPlace hlPlace) {
		setModel(hlPlace);
	}

	public HLPlace getHLPlace() {
		return (HLPlace) getModel();
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getModelSourceConnections()
	 */
	protected List getModelSourceConnections() {
		return getHLPlace().getIncoming();
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getModelTargetConnections()
	 */
	protected List getModelTargetConnections() {
		return getHLPlace().getOutgoing();
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#getModelChildren()
	 */
	@Override
	protected List getModelChildren() {
		return getHLPlace().getTokens();
	}

	/**
	 * Token-Figure dem HLPlace-Figure hinzufgen
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#addChildVisual(org.eclipse.gef.EditPart,
	 *      int)
	 */
	@Override
	protected void addChildVisual(EditPart childEditPart, int index) {
		int maxChildrenNumber = getChildren().size();
		getHLPlaceFigure().addToE(((TokenEditPart) childEditPart).getFigure(),
				index, maxChildrenNumber);
	}

	/**
	 * Token-Figure vom HLPlace-Figure entfernen
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#removeChildVisual(org.eclipse.gef.EditPart)
	 */
	@Override
	protected void removeChildVisual(EditPart childEditPart) {
		getHLPlaceFigure().removeChildFromE(
				((TokenEditPart) childEditPart).getFigure());
	}

	/**
	 * Returns the Figure of this, as a client type figure.
	 * 
	 * @return Figure as a HLPlaceFigure.
	 */
	protected HLPlaceFigure getPlaceFigure() {
		return (HLPlaceFigure) getFigure();
	}

	/**
	 *  
	 * @see org.eclipse.emf.common.notify.Adapter#notifyChanged(org.eclipse.emf.common.notify.Notification)
	 */
	public void notifyChanged(Notification notification) {
		int featureId = notification.getFeatureID(RonmodelPackage.class);
		switch (featureId) {
		case RonmodelPackage.HL_NODE__OUTGOING:
			refreshTargetConnections();
			break;
		case RonmodelPackage.HL_NODE__INCOMING:
			refreshSourceConnections();
			break;
		case RonmodelPackage.HL_PLACE__TOKENS:
			refreshChildren();
		default:
			refreshVisuals();
			break;
		}
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		// install the edit policy to handle connection creation
		installEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE,
				new HLNodeEditPolicy());
		// install the edit policy to handle deletion
		installEditPolicy(EditPolicy.COMPONENT_ROLE,
				new HLNodeComponentEditPolicy());
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 */
	protected void refreshVisuals() {
		getPlaceFigure().setName(
				getHLPlace().getType() + ":" + getHLPlace().getName());
		Point loc = new Point(getHLPlace().getX(), getHLPlace().getY());
		Dimension size = new Dimension(-1, -1);
		Rectangle r = new Rectangle(loc, size);
		((GraphicalEditPart) getParent()).setLayoutConstraint(this,
				getFigure(), r);
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshChildren()
	 */
	@Override
	protected void refreshChildren() {
		super.refreshChildren();
		getHLPlaceFigure().removeAllChildrenFromE();
		List<TokenEditPart> token = getChildren();
		int children = token.size();
		int index = 0;
		for (TokenEditPart part : token) {
			getHLPlaceFigure().addToE(part.getFigure(), index++, children);
		}
		getFigure().validate();

	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#activate()
	 */
	@SuppressWarnings("unchecked")
	public void activate() {
		if (isActive())
			return;

		// start listening for changes in the model

		getHLPlace().eAdapters().add(this);
		super.activate();
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#deactivate()
	 */
	public void deactivate() {
		if (!isActive())
			return;

		// stop listening for changes in the model
		getHLPlace().eAdapters().remove(this);

		super.deactivate();
	}

	/**
	 * @see org.eclipse.emf.common.notify.Adapter#getTarget()
	 */
	public Notifier getTarget() {
		return target;
	}

	/** 
	 * @see org.eclipse.emf.common.notify.Adapter#isAdapterForType(java.lang.Object)
	 */
	public boolean isAdapterForType(Object type) {
		return type.equals(getModel().getClass());
	}

	/**
	 * Returns a Command if the appropriate request is passed. Checks the
	 * Commands handled by installed EditPolicies (none for this EditPart) as
	 * well.
	 * @see org.eclipse.gef.EditPart#getCommand(org.eclipse.gef.Request)
	 */
	@Override
	public Command getCommand(final Request request) {
		if (request.getType().equals(CreateTokenAction.ID)) {
			if (getHLPlace().getType().equals(HLPlaceType.NET_LITERAL)) {
				final ObjectNet newToken = RONModelManager.getFactory()
						.createObjectNet();
				return new CreateTokenCommand(getHLPlace(), newToken);
			} else if (getHLPlace().getType().equals(HLPlaceType.RULE_LITERAL)) {
				final Rule newToken = RONModelManager.getFactory().createRule();
				LHS lhs = RONModelManager.getFactory().createLHS();
				lhs.setName("lhs");
				newToken.setLhs(lhs);
				RHS rhs = RONModelManager.getFactory().createRHS();
				rhs.setName("rhs");
				newToken.setRhs(rhs);
				NAC nac1 = RONModelManager.getFactory().createNAC();
				nac1.setName("nac1");
				newToken.getNacs().add(nac1);
				return new CreateTokenCommand(getHLPlace(), newToken);
			}
		}
		return super.getCommand(request);
	}

	/** 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#understandsRequest(org.eclipse.gef.Request)
	 */
	@Override
	public boolean understandsRequest(Request req) {
		if (req.getType().equals(CreateTokenAction.ID)) {
			return true;
		}
		return super.understandsRequest(req);
	}

	/** 
	 * @see org.eclipse.emf.common.notify.Adapter#setTarget(org.eclipse.emf.common.notify.Notifier)
	 */
	public void setTarget(Notifier newTarget) {
		target = newTarget;
	}

	/**
	 * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	 */
	public Object getAdapter(Class key) {
		if (IPropertySource.class == key) {
			return getPropertySource();
		}
		return super.getAdapter(key);
	}

	/**
	 * @see com.ibm.itso.sal330r.gefdemo.edit.NetworkElementEditPart#getPropertySource()
	 */
	protected IPropertySource getPropertySource() {
		if (propertySource == null) {
			propertySource = new HLPlacePropertySource(getHLPlace());
		}
		return propertySource;

	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	protected IFigure createFigure() {
		HLPlaceFigure figure = new HLPlaceFigure();

		return figure;
	}
}
