package roneditor.ronview.edit.parts;

import java.util.List;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.NodeEditPart;
import org.eclipse.ui.views.properties.IPropertySource;

import roneditor.ronview.edit.policies.HLNodeComponentEditPolicy;
import roneditor.ronview.edit.policies.HLNodeEditPolicy;
import roneditor.ronview.figures.HLTransitionFigure;
import roneditor.ronview.model.HLTransitionPropertySource;
import ronmodel.HLTransition;
import ronmodel.RonmodelPackage;

/**
 * EditPart for a {@link HLTransition}.
 */
public class HLTransitionEditPart extends AbstractHLNodeEditPart implements
		NodeEditPart, Adapter {
	private IPropertySource propertySource = null;

	private Notifier target;

	/**
	 * Creates a new {@link HLTransitionEditPart} instance.
	 * 
	 * @param hlTransition
	 */
	public HLTransitionEditPart(HLTransition hlTransition) {
		setModel(hlTransition);
	}

	public HLTransition getHLTransition() {
		return (HLTransition) getModel();
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getModelSourceConnections()
	 */
	protected List getModelSourceConnections() {
		return getHLTransition().getIncoming();
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getModelTargetConnections()
	 */
	protected List getModelTargetConnections() {
		return getHLTransition().getOutgoing();
	}

	/**
	 * Returns the {@link HLTransitionFigure}
	 * 
	 * @return Figure as a TransitionFigure.
	 */
	protected HLTransitionFigure getHLTransitionFigure() {
		return (HLTransitionFigure) getFigure();
	}

	/**
	 * @see org.eclipse.emf.common.notify.Adapter#notifyChanged(org.eclipse.emf.common.notify.Notification)
	 */
	public void notifyChanged(Notification notification) {
		int featureId = notification.getFeatureID(RonmodelPackage.class);
		switch (featureId) {
		case RonmodelPackage.HL_TRANSITION__OUTGOING:
			refreshVisuals();
			refreshTargetConnections();
			break;
		case RonmodelPackage.HL_TRANSITION__INCOMING:
			refreshVisuals();
			refreshSourceConnections();
			break;
		default:
			refreshVisuals();
			break;
		}
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		// install the edit policy to handle connection creation
		installEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE,
				new HLNodeEditPolicy());
		// install the edit policy to handle deletion
		installEditPolicy(EditPolicy.COMPONENT_ROLE,
				new HLNodeComponentEditPolicy());
	}

	/** 
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 */
	protected void refreshVisuals() {
		getHLTransitionFigure().setName(getHLTransition().getName());
		getHLTransitionFigure().setKind(getHLTransition().getKind());
		Point loc = new Point(getHLTransition().getX(), getHLTransition()
				.getY());
		Dimension size = new Dimension(-1, -1);
		Rectangle r = new Rectangle(loc, size);
		((GraphicalEditPart) getParent()).setLayoutConstraint(this,
				getFigure(), r);
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#activate()
	 */
	@SuppressWarnings("unchecked")
	public void activate() {
		if (isActive())
			return;

		// start listening for changes in the model
		((Notifier) getHLTransition()).eAdapters().add(this);

		super.activate();
	}

	/** 
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#deactivate()
	 */
	public void deactivate() {
		if (!isActive())
			return;

		// stop listening for changes in the model
		((Notifier) getHLTransition()).eAdapters().remove(this);

		super.deactivate();
	}

	/** 
	 * @see org.eclipse.emf.common.notify.Adapter#getTarget()
	 */
	public Notifier getTarget() {
		return target;
	}

	/**
	 * @see org.eclipse.emf.common.notify.Adapter#isAdapterForType(java.lang.Object)
	 */
	public boolean isAdapterForType(Object type) {
		return type.equals(getModel().getClass());
	}

	/**
	 * @see org.eclipse.emf.common.notify.Adapter#setTarget(org.eclipse.emf.common.notify.Notifier)
	 */
	public void setTarget(Notifier newTarget) {
		target = newTarget;
	}

	/**
	 * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	 */
	public Object getAdapter(Class key) {
		if (IPropertySource.class == key) {
			return getPropertySource();
		}
		return super.getAdapter(key);
	}

	/**
	 * @see com.ibm.itso.sal330r.gefdemo.edit.NetworkElementEditPart#getPropertySource()
	 */
	protected IPropertySource getPropertySource() {
		if (propertySource == null) {
			propertySource = new HLTransitionPropertySource(getHLTransition());
		}
		return propertySource;

	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	protected IFigure createFigure() {
		HLTransitionFigure figure = new HLTransitionFigure();
		return figure;
	}
}
