package roneditor.ronview.edit.parts;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Ellipse;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.MouseEvent;
import org.eclipse.draw2d.MouseListener;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.NodeEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.commands.Command;
import org.eclipse.ui.views.properties.IPropertySource;

import roneditor.IRonEditorConstants;
import roneditor.AbstractMultiViewerPage.MultiViewerPageViewer;
import roneditor.edit.policies.TokenComponentEditPolicy;
import roneditor.editor.RONEditor;
import roneditor.model.ObjectNetPropertySource;
import roneditor.model.RulePropertySource;
import roneditor.ronview.commands.SelectTokenCommand;
import ronmodel.ObjectNet;
import ronmodel.Rule;
import ronmodel.Token;

/**
 * EditPart for a {@link Token}.
 */
public class TokenEditPart extends AbstractHLNodeEditPart implements
		NodeEditPart, Adapter {
	public static final String MOUSE_PRESSED = "Mouse_Pressed";

	private IPropertySource propertySource = null;

	private Notifier target;

	/**
	 * Creates a new TokenEditPart instance.
	 * 
	 * @param token
	 */
	public TokenEditPart(Token token) {
		setModel(token);
	}

	public Token getToken() {
		return (Token) getModel();
	}

	@Override
	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.COMPONENT_ROLE,
				new TokenComponentEditPolicy());

	}

	/**
	 * This handles the direct edit and open request sent by the SelectionTool.
	 * @see org.eclipse.gef.EditPart#performRequest(org.eclipse.gef.Request)
	 */
	@Override
	public void performRequest(final Request req) {
		final Object type = req.getType();
		if (type.equals(RequestConstants.REQ_OPEN)) {
			if (getToken() instanceof ObjectNet) {
				openObjectNetView();
			} else {
				openRuleView();
			}
		}
	}

	/**
	 * Opens a View for editing the object net that is this editpart's model.
	 */
	private void openObjectNetView() {
		((RONEditor) ((MultiViewerPageViewer) getViewer()).getHostPage()
				.getEditor()).showView(IRonEditorConstants.objectNetViewID,
				(EObject) getToken());
	}

	/**
	 * Opens a View for editing the rule that is this editpart's model.
	 */
	private void openRuleView() {
		((RONEditor) ((MultiViewerPageViewer) getViewer()).getHostPage()
				.getEditor()).showView(IRonEditorConstants.ruleViewID,
				(EObject) getToken());
	}

	/** 
	 * @see org.eclipse.emf.common.notify.Adapter#notifyChanged(org.eclipse.emf.common.notify.Notification)
	 */
	public void notifyChanged(Notification notification) {
		refreshVisuals();
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 */
	protected void refreshVisuals() {
		Label tooltiplabel = (Label) getFigure().getToolTip();
		tooltiplabel.setText(getToken().getName());
		if (getToken().isSelected()) {
			getFigure().setBackgroundColor(ColorConstants.red);
		} else {
			getFigure().setBackgroundColor(ColorConstants.black);
		}
		getFigure().repaint();
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#activate()
	 */
	@SuppressWarnings("unchecked")
	public void activate() {
		if (isActive())
			return;

		// start listening for changes in the model

		getToken().eAdapters().add(this);
		super.activate();
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#deactivate()
	 */
	public void deactivate() {
		if (!isActive())
			return;

		// stop listening for changes in the model
		getToken().eAdapters().remove(this);
		super.deactivate();
	}

	/**
	 * @see org.eclipse.emf.common.notify.Adapter#getTarget()
	 */
	public Notifier getTarget() {
		return target;
	}

	/**
	 * @see org.eclipse.emf.common.notify.Adapter#isAdapterForType(java.lang.Object)
	 */
	public boolean isAdapterForType(Object type) {
		return type.equals(getModel().getClass());
	}

	/** 
	 * @see org.eclipse.emf.common.notify.Adapter#setTarget(org.eclipse.emf.common.notify.Notifier)
	 */
	public void setTarget(Notifier newTarget) {
		target = newTarget;
	}

	/**
	 * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	 */
	public Object getAdapter(Class key) {
		if (IPropertySource.class == key) {
			return getPropertySource();
		}
		return super.getAdapter(key);
	}

	/**
	 * @see com.ibm.itso.sal330r.gefdemo.edit.NetworkElementEditPart#getPropertySource()
	 */
	protected IPropertySource getPropertySource() {

		if (propertySource == null) {
			if (getToken() instanceof ObjectNet) {
				propertySource = new ObjectNetPropertySource(
						(ObjectNet) getToken());
			} else {
				propertySource = new RulePropertySource((Rule) getToken());
			}
		}
		return propertySource;

	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	protected IFigure createFigure() {
		Ellipse figure = new Ellipse();
		figure.setSize(12, 12);
		figure.setBackgroundColor(ColorConstants.black);
		figure.setOpaque(true);
		figure.setToolTip(new Label(getToken().getName()));
		figure.addMouseListener(new MouseListener.Stub() {
			@Override
			public void mousePressed(MouseEvent me) {
				new SelectTokenCommand(getToken()).execute();
			}
		});
		return figure;
	}
}
