package roneditor.ronview.model;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.Map;
import java.util.Queue;
import java.util.Vector;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.util.EcoreUtil;

import roneditor.objectnetview.model.ObjectNetModelUtil;
import ronmodel.HLArc;
import ronmodel.HLNode;
import ronmodel.HLPlace;
import ronmodel.HLPlaceType;
import ronmodel.HLTransition;
import ronmodel.ONArc;
import ronmodel.ONNode;
import ronmodel.ObjectNet;
import ronmodel.RonmodelFactory;
import ronmodel.RonmodelPackage;
import ronmodel.Token;
/**
 * in dieser Klasse werden hilfsmethoden implementiert, die auf das RONModell 
 * zugreifen.
 */
public class RONModelUtil {
	/**
	 * 
	 * @param source the {@link HLNode sourceNode}
	 * @param target the {@link HLNode targetNode}
	 * @return {@link HLArc} from source->target or null
	 */
	public static HLArc getHLArc(HLNode source, HLNode target) {
		EList outgoing = source.getOutgoing();
		HLArc hlArc = null;
		for (Object object : outgoing) {
			hlArc = (HLArc) object;
			if (hlArc.getTarget() == target)
				return hlArc;
		}
		return null;
	}
	/**
	 * 
	 * @param transition the {@link HLTransition}
	 * @return alle ObjectNetPlaces im Vorbereich
	 */
	public static Collection<HLPlace> getSourceNetPlaces(HLTransition transition) {
		Collection<HLPlace> sourcePlaces = new HashSet<HLPlace>();
		HLArc arc = null;
		HLPlace place = null;
		for (Object o : transition.getIncoming()) {
			arc = (HLArc) o;
			place = (HLPlace) arc.getSource();
			if (place.getType() == HLPlaceType.NET_LITERAL) {
				sourcePlaces.add(place);
			}
		}
		return sourcePlaces;
	}
	/**
	 * 
	 * @param transition the {@link HLTransition}
	 * @return alle RulePlaces im Vorbereich
	 */
	public static Collection<HLPlace> getSourceRulePlaces(
			HLTransition transition) {
		Collection<HLPlace> sourcePlaces = new HashSet<HLPlace>();
		HLArc arc = null;
		HLPlace place = null;
		for (Object o : transition.getIncoming()) {
			arc = (HLArc) o;
			place = (HLPlace) arc.getSource();
			if (place.getType() == HLPlaceType.RULE_LITERAL) {
				sourcePlaces.add(place);
			}
		}
		return sourcePlaces;
	}
	/**
	 * 
	 * @param transition the {@link HLTransition}
	 * @return alle ObjectNetPlaces im Nachbereich
	 */
	public static Collection<HLPlace> getTargetNetPlaces(HLTransition transition) {
		Collection<HLPlace> targetPlaces = new HashSet<HLPlace>();
		HLArc arc = null;
		HLPlace place = null;
		for (Object o : transition.getOutgoing()) {
			arc = (HLArc) o;
			place = (HLPlace) arc.getTarget();
			if (place.getType() == HLPlaceType.NET_LITERAL) {
				targetPlaces.add(place);
			}
		}
		return targetPlaces;
	}
	/**
	 * 
	 * @param transition the {@link HLTransition}
	 * @return alle RulePlaces im Nachbereich
	 */
	public static Collection<HLPlace> getTargetRulePlaces(
			HLTransition transition) {
		Collection<HLPlace> targetPlaces = new HashSet<HLPlace>();
		HLArc arc = null;
		HLPlace place = null;
		for (Object o : transition.getOutgoing()) {
			arc = (HLArc) o;
			place = (HLPlace) arc.getTarget();
			if (place.getType() == HLPlaceType.RULE_LITERAL) {
				targetPlaces.add(place);
			}
		}
		return targetPlaces;
	}
	/**
	 * 
	 * @param place the place
	 * @return random Token from this place 
	 */
	public static Token getRandomToken(HLPlace place) {
		int index = (int) (Math.random() * place.getTokens().size());
		return (Token) place.getTokens().get(index);
	}
	/**
	 * 
	 * @param objectNets ObjectNets die vereinigt werden
	 * @return ergebnis ObjectNet nach der Vereinigung
	 */
	public static ObjectNet union(Collection<Token> objectNets) {
		RonmodelFactory factory = RonmodelPackage.eINSTANCE
				.getRonmodelFactory();
		ObjectNet objectNet = factory.createObjectNet();
		objectNet.setName("new Net");
		ObjectNet copy = null;
		Collection<ONArc> arcs = new HashSet<ONArc>();
		Collection<ONNode> nodes = new HashSet<ONNode>();

		for (Token token : objectNets) {
			if (token instanceof ObjectNet) {
				ObjectNet net = (ObjectNet) token;
				copy = (ObjectNet) EcoreUtil.copy(net);
				nodes.addAll(copy.getNodes());
				arcs.addAll(copy.getArcs());
			}

		}
		for (ONNode node : nodes) {
			node.setObjectNet(objectNet);
		}
		for (ONArc arc : arcs) {
			arc.setObjectNet(objectNet);
		}
		return objectNet;
	}
	/**
	 * 
	 * @param hlPlace the {@link HLPlace}
	 * @return alle ObjectNets, die mindestens eine aktive Transition haben
	 */
	public static Vector<ObjectNet> getActiveObjectNets(HLPlace hlPlace) {
		Vector<ObjectNet> activeObjectNets = new Vector<ObjectNet>();
		if (hlPlace.getType() == HLPlaceType.NET_LITERAL) {
			for (Object token : hlPlace.getTokens()) {
				ObjectNet objectNet = (ObjectNet) token;
				if (!ObjectNetModelUtil.getActiveONTransitions(objectNet)
						.isEmpty()) {
					activeObjectNets.add(objectNet);
				}
			}
		}
		return activeObjectNets;
	}
	/**
	 * 
	 * @param objectNet the {@link ObjectNet}
	 * @return ergebnis ObjectNets nach dem Split
	 */
	public static Collection<ObjectNet> split(ObjectNet objectNet) {
		Map<Object, Integer> components = new HashMap<Object, Integer>();

		for (Object node : objectNet.getNodes()) {
			components.put(node, 0);
		}
		for (Object arc : objectNet.getArcs()) {
			components.put(arc, 0);
		}

		// Breitensuche
		int currentComponent = 1;
		for (Object o : objectNet.getNodes()) {
			ONNode node = (ONNode) o;
			if (components.get(node) == 0) {
				Queue<ONNode> queue = new LinkedList<ONNode>();
				queue.offer(node);
				components.put(node, currentComponent);
				while (!queue.isEmpty()) {
					node = queue.poll();
					for (Object in : node.getIncoming()) {
						ONArc inputArc = (ONArc) in;
						if (components.get(inputArc) == 0) {
							components.put(inputArc, currentComponent);
						}
						if (components.get(inputArc.getSource()) == 0) {
							queue.offer(inputArc.getSource());
							components.put(inputArc.getSource(),
									currentComponent);
						}
					}
					for (Object out : node.getOutgoing()) {
						ONArc outputArc = (ONArc) out;
						if (components.get(outputArc) == 0) {
							components.put(outputArc, currentComponent);
						}
						if (components.get(outputArc.getTarget()) == 0) {
							queue.offer(outputArc.getTarget());
							components.put(outputArc.getTarget(),
									currentComponent);
						}

					}
				}
				currentComponent++;
			}
		}
		RonmodelFactory factory = RonmodelPackage.eINSTANCE
				.getRonmodelFactory();
		Map<Integer, ObjectNet> subNets = new HashMap<Integer, ObjectNet>();
		for (int i = 1; i < currentComponent; i++) {
			ObjectNet subNet = factory.createObjectNet();
			subNet.setName("subNet " + i);
			subNets.put(i, subNet);
		}
		for (Object o : components.keySet()) {
			if (o instanceof ONNode) {
				ONNode onNode = (ONNode) o;
				onNode.setObjectNet(subNets.get(components.get(o)));
			}
		}
		for (Object o : components.keySet()) {
			if (o instanceof ONArc) {
				ONArc onArc = (ONArc) o;
				onArc.setObjectNet(subNets.get(components.get(o)));
			}
		}

		return subNets.values();
	}
	/**
	 * 
	 * @param hlPlace the {@link HLPlace}
	 * @return alle selektierte Tokens aus hlPlace
	 */
	public static Collection<Token> getSelectedTokens(HLPlace hlPlace) {
		Collection<Token> selectedTokens = new HashSet<Token>();
		for (Object o : hlPlace.getTokens()) {
			Token token = (Token) o;
			if (token.isSelected()) {
				selectedTokens.add(token);
			}
		}
		return selectedTokens;
	}
}
