package roneditor.ruleview.model;

import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import org.eclipse.emf.common.util.EList;

import roneditor.objectnetview.model.ObjectNetModelCreationFactory;
import ronmodel.Codomain;
import ronmodel.LHS;
import ronmodel.Mapping;
import ronmodel.ONArc;
import ronmodel.ONNode;
import ronmodel.ONPlace;
import ronmodel.ONTransition;
/**
 * diese Klasse enthlt hilfsmethoden, die auf das Modell 
 * zugreifen
 */
public class RuleModelUtil {
	/**
	 * @return Mapping zwichwen source und target wenn existiert,
	 * null sonst
	 */
	public static Mapping getMapping(ONNode source, ONNode target) {
		EList outgoingMapping = source.getOutgoingMappings();
		Mapping mapping = null;
		for (Object object : outgoingMapping) {
			mapping = (Mapping) object;
			if (mapping.getTarget() == target)
				return mapping;
		}
		return null;
	}

	/**
	 * @return alle {@link ONNode onNodes}, die sources ein gegebene Menge 
	 * von {@link ONArc arcs} sind
	 */
	public static List<ONNode> getSourceNodes(List arcs) {
		List<ONNode> nodes = new LinkedList<ONNode>();
		ONArc arc = null;
		for (Object object : arcs) {
			if (object instanceof ONArc) {
				arc = (ONArc) object;
				nodes.add(arc.getSource());
			}
		}
		return nodes;
	}

	/**
	 * @return alle {@link ONNode onNodes}, die targets ein gegebene Menge 
	 * von {@link ONArc arcs} sind
	 */
	public static List<ONNode> getTargetNodes(List arcs) {
		List<ONNode> nodes = new LinkedList<ONNode>();
		ONArc arc = null;
		for (Object object : arcs) {
			if (object instanceof ONArc) {
				arc = (ONArc) object;
				nodes.add(arc.getTarget());
			}
		}
		return nodes;
	}

	/**
	 * @return eingehende mapping zum target wenn existiert,
	 * null sonst
	 */
	public static Mapping getIncomingMapping(ONNode target) {
		Mapping mapping = null;
		if (target != null) {
			if (!target.getIncomingMappings().isEmpty()) {
				mapping = (Mapping) target.getIncomingMappings().get(0);
				return mapping;
			}
		}
		return null;

	}

	/**
	 * @return ausgehende mapping vom source zu codomain wenn existiert,
	 * null sonst
	 */
	public static Mapping getOutgoingMapping(ONNode source, Codomain codomain) {
		Mapping mapping = null;
		if (source != null) {
			for (Object o : source.getOutgoingMappings()) {
				mapping = (Mapping) o;
				if (mapping.getCodomain() == codomain) {
					return mapping;
				}
			}
		}
		return null;
	}

	/**
	 * @return alle mapping fr transition, die fehlerhaft sind
	 */
	public static Set<Mapping> getErrorMapping(ONTransition transition) {
		HashSet<Mapping> errorMappings = new HashSet<Mapping>();
		Mapping mapping = null;
		if (transition != null) {
			for (Object o : transition.getIncomingMappings()) {
				mapping = (Mapping) o;
				if (!checkMapping((ONTransition) mapping.getSource(),
						transition)) {
					errorMappings.add(mapping);
				}
			}
			for (Object o : transition.getOutgoingMappings()) {
				mapping = (Mapping) o;
				if (!checkMapping(transition, (ONTransition) mapping
						.getTarget())) {
					errorMappings.add(mapping);
				}
			}
		}
		return errorMappings;
	}

	/**
	 * @return copy von onNode
	 */
	public static ONNode copy(ONNode onNode) {
		ONNode result = null;
		ObjectNetModelCreationFactory factory = null;
		if (onNode != null) {
			if (onNode instanceof ONPlace) {
				ONPlace place = (ONPlace) onNode;
				factory = new ObjectNetModelCreationFactory(ONPlace.class);
				ONPlace copy = (ONPlace) factory.getNewObject();
				copy.setName(place.getName());
				copy.setTokens(place.getTokens());
				copy.setMappingID(place.getMappingID());
				result = copy;
			} else if (onNode instanceof ONTransition) {
				ONTransition transition = (ONTransition) onNode;
				factory = new ObjectNetModelCreationFactory(ONTransition.class);
				ONTransition copy = (ONTransition) factory.getNewObject();
				copy.setName(transition.getName());
				copy.setMappingID(transition.getMappingID());
				result = copy;
			}
			result.setX(onNode.getX());
			result.setY(onNode.getY());
		}
		return result;
	}

	/**
	 * @return copy von onArc
	 */
	public static ONArc copy(ONArc onArc) {
		ONArc copy = null;
		ObjectNetModelCreationFactory factory = null;
		if (onArc != null) {
			factory = new ObjectNetModelCreationFactory(ONArc.class);
			copy = (ONArc) factory.getNewObject();
			copy.setWeight(onArc.getWeight());
			copy.setSourceAnchorID(onArc.getSourceAnchorID());
			copy.setTargetAnchorID(onArc.getTargetAnchorID());
		}
		return copy;
	}

	/**
	 * @return true wenn das Mapping zwischen 2 Transitionen nicht fehlerhaft ist,
	 * false sonst
	 */
	public static boolean checkMapping(ONTransition t1, ONTransition t2) {
		if (t1 == null || t2 == null || !(t1.getObjectNet() instanceof LHS)
				|| !(t2.getObjectNet() instanceof Codomain))
			return false;
		if (t1.getIncoming().size() != t2.getIncoming().size())
			return false;
		if (t1.getOutgoing().size() != t2.getOutgoing().size())
			return false;

		ONArc arc2 = null;
		ONPlace place1 = null;
		ONPlace place2 = null;
		Mapping mapping = null;
		for (Object o : t2.getIncoming()) {
			arc2 = (ONArc) o;
			place2 = (ONPlace) arc2.getSource();
			if (place2.getIncomingMappings().size() != 1) {
				return false;
			}
			mapping = (Mapping) place2.getIncomingMappings().get(0);
			place1 = (ONPlace) mapping.getSource();
			if (!getSourceNodes(t1.getIncoming()).contains(place1)) {
				return false;
			}
		}
		for (Object o : t2.getOutgoing()) {
			arc2 = (ONArc) o;
			place2 = (ONPlace) arc2.getTarget();
			if (place2.getIncomingMappings().size() != 1) {
				return false;
			}
			mapping = (Mapping) place2.getIncomingMappings().get(0);
			place1 = (ONPlace) mapping.getSource();
			if (!getTargetNodes(t1.getOutgoing()).contains(place1)) {
				return false;
			}
		}
		return true;
	}

	/**
	 * @return true wenn das Mapping nicht fehlerhaft ist,
	 * false sonst
	 */
	public static boolean checkMapping(Mapping mapping) {
		if (mapping == null
				|| mapping.getSource().getClass() != mapping.getTarget()
						.getClass())
			return false;
		if (mapping.getSource() instanceof ONPlace)
			return true;
		ONTransition t1 = (ONTransition) mapping.getSource();
		ONTransition t2 = (ONTransition) mapping.getTarget();
		return checkMapping(t1, t2);
	}

	/**
	 * @return true wenn das Mapping gelscht werden kann,
	 * false sonst
	 */
	public static boolean canDelete(Mapping mapping) {
		if (mapping.getSource() instanceof ONTransition)
			return true;

		ONPlace target = (ONPlace) mapping.getTarget();
		ONTransition transition = null;
		for (Object o : target.getOutgoing()) {
			transition = (ONTransition) ((ONArc) o).getTarget();
			if (!transition.getIncomingMappings().isEmpty()) {
				return false;
			}
		}
		for (Object o : target.getIncoming()) {
			transition = (ONTransition) ((ONArc) o).getSource();
			if (!transition.getIncomingMappings().isEmpty()) {
				return false;
			}
		}
		return true;
	}

	/*
	 * 
	 */
	public static Set<ONTransition> getTransitions(ONPlace place) {
		HashSet<ONTransition> transitions = new HashSet<ONTransition>();
		ONArc arc = null;

		for (Object o : place.getIncoming()) {
			arc = (ONArc) o;
			transitions.add((ONTransition) arc.getSource());
		}
		for (Object o : place.getOutgoing()) {
			arc = (ONArc) o;
			transitions.add((ONTransition) arc.getTarget());
		}
		return transitions;
	}
}
